from numpy import *
import numba
import math, cmath

##################### not yet correct ###############################

@numba.jit(numba.complex128[:, :](numba.complex128[:, :], numba.complex128[:, :]),
           nopython=True, cache=True)
def dot4(A, B):
    D=empty((4, 4), dtype=complex128)
    D[0, 0]=(A[0, 0]*B[0, 0]+A[0, 1]*B[1, 0]+A[0, 2]*B[2, 0]+
             A[0, 3]*B[3, 0])
    D[0, 1]=(A[0, 0]*B[0, 1]+A[0, 1]*B[1, 1]+A[0, 2]*B[2, 1]+
             A[0, 3]*B[3, 1])
    D[0, 2]=(A[0, 0]*B[0, 2]+A[0, 1]*B[1, 2]+A[0, 2]*B[2, 2]+
             A[0, 3]*B[3, 2])
    D[0, 3]=(A[0, 0]*B[0, 3]+A[0, 1]*B[1, 3]+A[0, 2]*B[2, 3]+
             A[0, 3]*B[3, 3])

    D[1, 0]=(A[1, 0]*B[0, 0]+A[1, 1]*B[1, 0]+A[1, 2]*B[2, 0]+
             A[1, 3]*B[3, 0])
    D[1, 1]=(A[1, 0]*B[0, 1]+A[1, 1]*B[1, 1]+A[1, 2]*B[2, 1]+
             A[1, 3]*B[3, 1])
    D[1, 2]=(A[1, 0]*B[0, 2]+A[1, 1]*B[1, 2]+A[1, 2]*B[2, 2]+
             A[1, 3]*B[3, 2])
    D[1, 3]=(A[1, 0]*B[0, 3]+A[1, 1]*B[1, 3]+A[1, 2]*B[2, 3]+
             A[1, 3]*B[3, 3])

    D[2, 0]=(A[2, 0]*B[0, 0]+A[2, 1]*B[1, 0]+A[2, 2]*B[2, 0]+
             A[2, 3]*B[3, 0])
    D[2, 1]=(A[2, 0]*B[0, 1]+A[2, 1]*B[1, 1]+A[2, 2]*B[2, 1]+
             A[2, 3]*B[3, 1])
    D[2, 2]=(A[2, 0]*B[0, 2]+A[2, 1]*B[1, 2]+A[2, 2]*B[2, 2]+
             A[2, 3]*B[3, 2])
    D[2, 3]=(A[2, 0]*B[0, 3]+A[2, 1]*B[1, 3]+A[2, 2]*B[2, 3]+
             A[2, 3]*B[3, 3])

    D[3, 0]=(A[3, 0]*B[0, 0]+A[3, 1]*B[1, 0]+A[3, 2]*B[2, 0]+
             A[3, 3]*B[3, 0])
    D[3, 1]=(A[3, 0]*B[0, 1]+A[3, 1]*B[1, 1]+A[3, 2]*B[2, 1]+
             A[3, 3]*B[3, 1])
    D[3, 2]=(A[3, 0]*B[0, 2]+A[3, 1]*B[1, 2]+A[3, 2]*B[2, 2]+
             A[3, 3]*B[3, 2])
    D[3, 3]=(A[3, 0]*B[0, 3]+A[3, 1]*B[1, 3]+A[3, 2]*B[2, 3]+
             A[3, 3]*B[3, 3])

    return D

@numba.jit(numba.float64[:, :](numba.float64[:], numba.complex128[:], numba.complex128[:],
                               numba.float64[:], numba.float64[:], numba.float64[:]),
           nopython=True, parallel=True, cache=True)
def ReflNBSigma(Q, Vp, Vm, d, M_ang, sigma):
    '''A quicker implementation than the ordinary slow implementaion in Refl
    Calculates spin-polarized reflectivity according to S.J. Blundell
        and J.A.C. Bland Phys rev. B. vol 46 3391 (1992)
        The algorithm assumes that the first element in the arrays represents
        the substrate and the last the ambient layer.
        Input parameters:   Q : Scattering vector in reciprocal
                                angstroms Q=4*pi/lambda *sin(theta)
                            Vp: Neutron potential for spin up
                            Vm: Neutron potential for spin down
                            d: layer thickness
                            M_ang: Angle of the magnetic
                                    moment(radians!) M_ang=0 =>M//neutron spin
                            sigma: The roughness of the upper interface.
                            return_int: Flag for returning the instensity, default=True. If False return the amplitudes.
        Returns:            (Ruu,Rdd,Rud,Rdu)
                            (up-up,down-down,up-down,down-up)
    '''
    Rout=empty((4, Q.shape[0]), dtype=float64)

    layers=Vp.shape[0]
    points=Q.shape[0]

    for qi in numba.prange(points):
        X=empty((4, 4), dtype=complex128)
        P=empty((4,), dtype=complex128)
        PX=empty((layers, 4, 4), dtype=complex128)

        # Assume first element=substrate and last=ambient!
        k_amb2=(Q[qi]/2.0)**2

        k_pi=cmath.sqrt(k_amb2-Vp[0])
        k_mi=cmath.sqrt(k_amb2-Vm[0])

        for lj in range(1, layers):
            # Wavevectors in the layers
            k_pj=cmath.sqrt(k_amb2-Vp[lj])
            k_mj=cmath.sqrt(k_amb2-Vm[lj])

            theta_diff=M_ang[lj]-M_ang[lj-1]

            ##### ass_X ####
            # Angular difference between the magnetization
            # Assemble the interface reflectivity matrix
            costd=math.cos(theta_diff/2.0)
            sintd=math.sin(theta_diff/2.0)
            X[0, 0]=costd*(k_pi+k_pj)/2./k_pj
            X[0, 1]=-costd*(k_pi-k_pj)/2./k_pj
            X[0, 2]=sintd*(k_pj+k_mi)/2./k_pj
            X[0, 3]=sintd*(k_pj-k_mi)/2./k_pj
            X[1, 0]=X[0, 1]  # -(costd*(k_pj1 - k_pj))/(2*k_pj)
            X[1, 1]=X[0, 0]  # (costd*(k_pj1 + k_pj))/(2*k_pj)
            X[1, 2]=X[0, 3]  # (sintd*(k_pj - k_mj1))/(2*k_pj)
            X[1, 3]=X[0, 2]  # (sintd*(k_pj + k_mj1))/(2*k_pj)
            X[2, 0]=-(sintd*(k_pi+k_mj))/(2.*k_mj)
            X[2, 1]=(sintd*(k_pi-k_mj))/(2.*k_mj)
            X[2, 2]=(costd*(k_mi+k_mj))/(2.*k_mj)
            X[2, 3]=-(costd*(k_mi-k_mj))/(2.*k_mj)
            X[3, 0]=X[2, 1]  # (sintd*(k_pj1 - k_mj))/(2*k_mj)
            X[3, 1]=X[2, 0]  # -(sintd*(k_pj1 + k_mj))/(2*k_mj)
            X[3, 2]=X[2, 3]  # -(costd*(k_mj1 - k_mj))/(2*k_mj)
            X[3, 3]=X[2, 2]  # (costd*(k_mj1 + k_mj))/(2*k_mj)

            ##### include_sigma #####
            sigma2=sigma[lj]**2/2.0
            X[0, 0]=X[0, 0]*cmath.exp(-(k_pj-k_pi)**2*sigma2)
            X[0, 1]=X[0, 1]*cmath.exp(-(k_pj+k_pi)**2*sigma2)
            X[0, 2]=X[0, 2]*cmath.exp(-(k_pj-k_mi)**2*sigma2)
            X[0, 3]=X[0, 3]*cmath.exp(-(k_pj+k_mi)**2*sigma2)
            X[1, 0]=X[0, 1]  # X[1,0]*w(k_pj + k_pj1, sigma2)
            X[1, 1]=X[0, 0]  # X[1,1]*w(k_pj - k_pj1, sigma2)
            X[1, 2]=X[0, 3]  # X[1,2]*w(k_pj + k_mj1, sigma2)
            X[1, 3]=X[0, 2]  # X[1,3]*w(k_pj - k_mj1, sigma2)
            X[2, 0]=X[2, 0]*cmath.exp(-(k_mj-k_pi)**2*sigma2)
            X[2, 1]=X[2, 1]*cmath.exp(-(k_mj+k_pi)**2*sigma2)
            X[2, 2]=X[2, 2]*cmath.exp(-(k_mj-k_mi)**2*sigma2)
            X[2, 3]=X[2, 3]*cmath.exp(-(k_mj+k_mi)**2*sigma2)
            X[3, 0]=X[2, 1]  # X[3,0]*w(k_mj + k_pj1, sigma)
            X[3, 1]=X[2, 0]  # X[3,1]*w(k_mj - k_pj1, sigma)
            X[3, 2]=X[2, 3]  # X[3,2]*w(k_mj + k_mj1, sigma)
            X[3, 3]=X[2, 2]  # X[3,3]*w(k_mj - k_mj1, sigma)

            ##### ass_P ####
            P[0]=cmath.exp(-1.0J*k_pj*d[lj])
            P[1]=1./P[0]  # exp(1.0J*k_p*d)
            P[2]=cmath.exp(-1.0J*k_mj*d[lj])
            P[3]=1./P[2]  # exp(1.0J*k_m*d)

            # Assemble the layer propagation matrices

            # Multiply the propagation matrices with the interface matrix
            for i in range(4):
                for j in range(4):
                    PX[lj, i, j]=P[i]*X[i, j]
            k_pi=k_pj
            k_mi=k_mj

        ##### ass_P ####
        M=PX[-2]
        for linv in range(layers-3):
            # Multiply up the sample matrix
            M=dot4(M, PX[layers-3-linv])
        M=dot4(X, M)

        denom=M[0, 0]*M[2, 2]-M[0, 2]*M[2, 0]
        Ruu=(M[1, 0]*M[2, 2]-M[1, 2]*M[2, 0])/denom
        Rud=(M[3, 0]*M[2, 2]-M[3, 2]*M[2, 0])/denom
        Rdu=(M[1, 2]*M[0, 0]-M[1, 0]*M[0, 2])/denom
        Rdd=(M[3, 2]*M[0, 0]-M[3, 0]*M[0, 2])/denom
        Rout[0, qi]=abs(Ruu)**2
        Rout[1, qi]=abs(Rdd)**2
        Rout[2, qi]=abs(Rud)**2
        Rout[3, qi]=abs(Rdu)**2
    return Rout

@numba.jit(numba.float64[:, :](numba.float64[:], numba.complex128[:], numba.complex128[:],
                               numba.float64[:], numba.float64[:]),
           nopython=True, parallel=True, cache=True)
def ReflNB(Q, Vp, Vm, d, M_ang):
    '''A quicker implementation than the ordinary slow implementaion in Refl
    Calculates spin-polarized reflectivity according to S.J. Blundell
        and J.A.C. Bland Phys rev. B. vol 46 3391 (1992)
        The algorithm assumes that the first element in the arrays represents
        the substrate and the last the ambient layer.
        Input parameters:   Q : Scattering vector in reciprocal
                                angstroms Q=4*pi/lambda *sin(theta)
                            Vp: Neutron potential for spin up
                            Vm: Neutron potential for spin down
                            d: layer thickness
                            M_ang: Angle of the magnetic
                                    moment(radians!) M_ang=0 =>M//neutron spin
                            sigma: The roughness of the upper interface.
                            return_int: Flag for returning the instensity, default=True. If False return the amplitudes.
        Returns:            (Ruu,Rdd,Rud,Rdu)
                            (up-up,down-down,up-down,down-up)
    '''
    Rout=empty((4, Q.shape[0]), dtype=float64)

    layers=Vp.shape[0]
    points=Q.shape[0]

    for qi in numba.prange(points):
        X=empty((4, 4), dtype=complex128)
        P=empty((4,), dtype=complex128)
        PX=empty((layers, 4, 4), dtype=complex128)

        # Assume first element=substrate and last=ambient!
        k_amb2=(Q[qi]/2.0)**2

        k_pi=cmath.sqrt(k_amb2-Vp[0])
        k_mi=cmath.sqrt(k_amb2-Vm[0])

        for lj in range(1, layers):
            # Wavevectors in the layers
            k_pj=cmath.sqrt(k_amb2-Vp[lj])
            k_mj=cmath.sqrt(k_amb2-Vm[lj])

            theta_diff=M_ang[lj]-M_ang[lj-1]

            ##### ass_X ####
            # Angular difference between the magnetization
            # Assemble the interface reflectivity matrix
            costd=math.cos(theta_diff/2.0)
            sintd=math.sin(theta_diff/2.0)
            X[0, 0]=costd*(k_pi+k_pj)/2./k_pj
            X[0, 1]=-costd*(k_pi-k_pj)/2./k_pj
            X[0, 2]=sintd*(k_pj+k_mi)/2./k_pj
            X[0, 3]=sintd*(k_pj-k_mi)/2./k_pj
            X[1, 0]=X[0, 1]  # -(costd*(k_pj1 - k_pj))/(2*k_pj)
            X[1, 1]=X[0, 0]  # (costd*(k_pj1 + k_pj))/(2*k_pj)
            X[1, 2]=X[0, 3]  # (sintd*(k_pj - k_mj1))/(2*k_pj)
            X[1, 3]=X[0, 2]  # (sintd*(k_pj + k_mj1))/(2*k_pj)
            X[2, 0]=-(sintd*(k_pi+k_mj))/(2.*k_mj)
            X[2, 1]=(sintd*(k_pi-k_mj))/(2.*k_mj)
            X[2, 2]=(costd*(k_mi+k_mj))/(2.*k_mj)
            X[2, 3]=-(costd*(k_mi-k_mj))/(2.*k_mj)
            X[3, 0]=X[2, 1]  # (sintd*(k_pj1 - k_mj))/(2*k_mj)
            X[3, 1]=X[2, 0]  # -(sintd*(k_pj1 + k_mj))/(2*k_mj)
            X[3, 2]=X[2, 3]  # -(costd*(k_mj1 - k_mj))/(2*k_mj)
            X[3, 3]=X[2, 2]  # (costd*(k_mj1 + k_mj))/(2*k_mj)

            ##### ass_P ####
            P[0]=cmath.exp(-1.0J*k_pj*d[lj])
            P[1]=1./P[0]  # exp(1.0J*k_p*d)
            P[2]=cmath.exp(-1.0J*k_mj*d[lj])
            P[3]=1./P[2]  # exp(1.0J*k_m*d)

            # Assemble the layer propagation matrices

            # Multiply the propagation matrices with the interface matrix
            for i in range(4):
                for j in range(4):
                    PX[lj, i, j]=P[i]*X[i, j]
            k_pi=k_pj
            k_mi=k_mj

        ##### ass_P ####
        M=PX[-2]
        for linv in range(layers-3):
            # Multiply up the sample matrix
            M=dot4(M, PX[layers-3-linv])
        M=dot4(X, M)

        denom=M[0, 0]*M[2, 2]-M[0, 2]*M[2, 0]
        Ruu=(M[1, 0]*M[2, 2]-M[1, 2]*M[2, 0])/denom
        Rud=(M[3, 0]*M[2, 2]-M[3, 2]*M[2, 0])/denom
        Rdu=(M[1, 2]*M[0, 0]-M[1, 0]*M[0, 2])/denom
        Rdd=(M[3, 2]*M[0, 0]-M[3, 0]*M[0, 2])/denom
        Rout[0, qi]=abs(Ruu)**2
        Rout[1, qi]=abs(Rdd)**2
        Rout[2, qi]=abs(Rud)**2
        Rout[3, qi]=abs(Rdu)**2
    return Rout

def Refl(Q, Vp, Vm, d, M_ang, sigma=None, return_int=True):
    if sigma is not None:
        return ReflNBSigma(Q, Vp.astype(complex128), Vm.astype(complex128), d, M_ang, sigma)
    else:
        return ReflNB(Q, Vp.astype(complex128), Vm.astype(complex128), d, M_ang)



'''<h1>Utilities for GenX and scattering</h1>
This library contains nice to have functions and classes. Most noteworthy
is the UserVars class. The scattering factor tables can also come handy.

<h2>Classes</h2>

<h3>UserVars</h3>
This class is used to contain user defined variables which is problem specific
and do not warrant a completly new model library.
It is contstructed as: <code>vars = UserVars()</code>.<br>
A new variable is created by calling the new_var method: 
<code>vars.new_var('my_var', 10.0)</code><br>
The methods in this class are:
<dl>
    <dt><code><b>__init__(self)</b></code></dt>
    <dd>Create a new variable container</dd>
    <dt><code><b>new_var(self, name, value)</b></code></dt>
    <dd>Create a new varaible with name [string] name and value value [float]
    </dd>
</dl>
The created varaibles are accesed through the as [Uservars].set[Name] and
[Uservars].get[Name]. In the example above this would read:
 <code>vars.setMy_var(20)</code> to set a value and 
<code>vars.getMy_vars()</code> to retrieve the value.


<h3>Scattering factor databases</h3>
The following databases exist in GenX. All databases are accesed through
<code>[name].[element]</code> for example: <code>fp.fe</code>

<h4>fp</h4>
The fp contains the scattering factors at Q = 0 for the 92 first elements up
to U in units of electrons or more precise Thompson scattering lengths. 
The data is taken from <a href = "http://henke.lbl.gov/optical_constants/asf.html">
http://henke.lbl.gov/optical_constants/asf.html</a>. These tables are also 
known as the henke tables. According to
cxros homepage: 
There are 500+ points on a uniform logarithmic mesh from 
10 to 30,000 eV with points added 0.1 eV above and 
below "sharp" absorption edges. For some elements data on a 
finer mesh includes structure around absorption edges. 
(Below 29 eV f1 is set equal to -9999.) 

<h4>bc</h4>
The bc data base contains the coherent scattering length for neutrons according
to the data published in Neutron News, Vol. 3, No. 3, 1992, pp.  29-37.
The data file is taken from the Dabax library compiled by esrf
<a href = "http://ftp.esrf.eu/pub/scisoft/xop2.3/DabaxFiles/">
http://ftp.esrf.eu/pub/scisoft/xop2.3/DabaxFiles/</a>.
To use isotopes just add "i" in front of the name i.e., bc.i57Fe.

<h4>f</h4>
The f database contains the <em>scattering vector dependent</em> isotropic
scattering factors in electrons or more precise Thompson scattering lengths.
The data is taken from the fp and the f0 database.

<h4>f0</h4>
The f0 database contains the <em>scattering vector dependent</em> isotropic
scattering factors in electrons or more precise Thompson scattering lengths.
The data is the so-called Croemer-Mann tables see:
International Tables vol. 4 or vol C; in vol. C refer to pg 500-502.
 Note, this is only good out to sin(theta)/lambda < 2.0 [Angstrom^-1].
The data is also fetched from the Dabax library at:
<a href = "http://ftp.esrf.eu/pub/scisoft/xop2.3/DabaxFiles/">
http://ftp.esrf.eu/pub/scisoft/xop2.3/DabaxFiles/</a>
This database is the non-dispersive (without resonant contribution).

<h4>fw</h4>
Same thing as f but scaled so that is can be used with a density
in g/cm<sup>3<sup>.

<h4>bw</h4>
Same thing as bc but scaled so that is can be used with a density
in g/cm<sup>3<sup>.
'''
import numpy as np
from .lib import scatteringlengths as sl
import os
from genx.gui_logging import iprint

_head, _tail=os.path.split(__file__)
# Look only after the file name and not the ending since
# the file ending can be pyc if compiled... 
__FILENAME__=_tail.split('.')[0]
# This assumes that plugin is under the current dir may need 
# changing
__MODULE_DIR__=_head
if __MODULE_DIR__=='':
    __MODULE_DIR__='.'

class UserVars:
    def __init__(self):
        pass

    def newVar(self, name, value):
        # name=name.lower()
        setattr(self, name, value)
        setattr(self, 'set'+name[0].upper()+name[1:], lambda value: setattr(self, name, value))
        setattr(self, 'get'+name[0].upper()+name[1:], lambda: getattr(self, name, value))

    def new_var(self, name, value):
        # name=name.lower()
        setattr(self, name, value)
        setattr(self, 'set'+name[0].upper()+name[1:], lambda value: setattr(self, name, value))
        setattr(self, 'get'+name[0].upper()+name[1:], lambda: getattr(self, name, value))

# ==============================================================================
# Now create default databases for scattering lengths and form factors
# for now only x-rays at single wavelengths
# The non-disersive part but angle dependent
__f0_dict__, __rho0_dict__=sl.load_f0dabax(__MODULE_DIR__+
                                           '/databases/f0_CromerMann.dat',
                                           create_rho=True)
# Workaround for the oxygen
__f0_dict__['o2m']=__f0_dict__['o2-.']
__rho0_dict__['o2m']=__rho0_dict__['o2-.']
f0=sl.ScatteringLength(__f0_dict__)
# Dispersive part at Q = 0
__lookup_fp__=sl.create_fp_lookup(__MODULE_DIR__+'/databases/f1f2_nist/')
__lookup_fp_old__=sl.create_fp_lookup(__MODULE_DIR__+'/databases/f1f2_cxro/')

def create_fp(wavelength):
    return sl.FormFactor(wavelength, __lookup_fp__)

def create_fp_old(wavelength):
    return sl.FormFactor(wavelength, __lookup_fp_old__)

fp=create_fp(1.54)
fp_old=create_fp_old(1.54)
# The total angle dependent form factor
__lookup_f__=sl.create_f_lookup(__lookup_fp__, __f0_dict__)
f=sl.FormFactor(1.54, __lookup_f__)
# The electrondensity of an atom
__lookup_rho__=sl.create_rho_lookup(__lookup_fp__, __rho0_dict__,
                                    __f0_dict__)
rho=sl.FormFactor(1.54, __lookup_rho__)
# The coherent scattering length for neutrons
__bc_dict__=sl.load_bdabax(__MODULE_DIR__+'/databases/DeBe_NeutronNews.dat')
bc=sl.ScatteringLength(__bc_dict__)
# print 'Loading atomic weights'
__w_dict__=sl.load_atomic_weights_dabax(__MODULE_DIR__+ \
                                        '/databases/AtomicWeights.dat')
# print 'Making bw dict'
__bw_dict__=sl.create_scatt_weight(__bc_dict__, __w_dict__)
# print 'Making bw scattering lengths'
bw=sl.ScatteringLength(__bw_dict__)
# print 'Making fw scattering lengths'
__lookup_fw__=sl.create_fw_lookup(__lookup_fp__, __w_dict__)

def create_fw(wavelength):
    return sl.FormFactor(wavelength, __lookup_fw__)

fw=create_fw(1.54)

__f_chantler_dict__=sl.read_dabax(__MODULE_DIR__+'/databases/f1f2_Chantler.dat')
__lookup_fpc__=sl.load_fdabax(__MODULE_DIR__+'/databases/f1f2_Chantler.dat')
fpc=sl.FormFactor(15.4, __lookup_fpc__)

__lookup_fdisp__=sl.create_fpdisp_lookup(__MODULE_DIR__+'/databases/f1f2_nist/')
fd=sl.Database()
object.__setattr__(fd, 'lookup_value', __lookup_fdisp__)

if __name__=='__main__':
    # MyVars=UserVars()
    # MyVars.newVar('a',3)
    # print 'f0.fe(0)', f0.fe(0)
    # fp.set_wavelength(1.54)
    # print 'fp.fe', fp.fe
    # f.set_wavelength(1.54)
    # print 'f.fe(0)', f.fe(0)
    # print 'f.fe2p(0)', f.fe2p(0)
    # fe = np.array(__f_chantler_dict__['fe'])
    # print fe.reshape(len(fe)/7, 7)
    ##print bc.fe
    # print fpc.Sn
    # print fp.Sn
    import time

    N=1000
    t1=time.time()
    for i in range(N):
        fd.Fe(100.0)
    t2=time.time()
    iprint("Dispersive database access time: ", (t2-t1)/N)
    t1=time.time()
    for i in range(N):
        ignore=fp.Fe
    t2=time.time()
    iprint("Normal database access time: ", (t2-t1)/N)



'''
Controller class for the differnetial evolution class diffev
Takes care of stopping and starting - output to the gui as well
as some input from dialog boxes.
Programmer Matts Bjorck
Last Changed 2009 05 12
'''
import wx, io, traceback
import wx.lib.newevent
from wx.lib.masked import NumCtrl

from . import diffev, fom_funcs
from . import filehandling as io
from .gui_logging import iprint
import numpy as np

# ==============================================================================
class SolverController:
    '''
    Class to take care of the GUI - solver interaction.
    Implements dialogboxes for setting parameters and controls
    for the solver routine. This is where the application specific
    code are used i.e. interfacing the optimization code to the GUI.
    '''

    def __init__(self, parent, config=None):
        # Create the optimizer we are using. In this case the standard
        # Differential evolution optimizer.
        self.optimizer=diffev.DiffEv()
        # Store the parent we need this to bind the different components to
        # the optimization algorithm.
        self.parent=parent
        self.config=config

        # Just storage of the starting values for the paramters before
        # the fit is started so the user can go back to the start values
        self.start_parameter_values=None
        # The level used for error bar calculations
        self.fom_error_bars_level=1.05

        # Setup the output functions.
        self.optimizer.set_text_output_func(self.TextOutput)
        self.optimizer.set_plot_output_func(self.PlotOutput)
        self.optimizer.set_parameter_output_func(self.ParameterOutput)
        self.optimizer.set_fitting_ended_func(self.FittingEnded)
        self.optimizer.set_autosave_func(self.AutoSave)

        self.parent.Bind(EVT_FITTING_ENDED, self.OnFittingEnded)

        # Now load the default configuration
        self.ReadConfig()

    def ReadConfig(self):
        '''ReadConfig(self) --> None
        
        Reads the parameter that should be read from the config file.
        And set the parameters in both the optimizer and this class.
        '''
        error_bars_level, save_all_evals=io.load_opt_config(self.optimizer, self.config)
        self.set_error_bars_level(error_bars_level)
        self.set_save_all_evals(save_all_evals)

    def WriteConfig(self):
        ''' WriteConfig(self) --> None
        
        Writes the current configuration of the solver to file.
        '''
        io.save_opt_config(self.optimizer, self.config, self.fom_error_bars_level, self.save_all_evals)

    def ParametersDialog(self, frame):
        '''ParametersDialog(self, frame) --> None
        
        Shows the Parameters dialog box to set the parameters for the solver.
        '''
        # Update the configuration if a model has been loaded after
        # the object have been created..
        self.ReadConfig()
        fom_func_name=self.parent.model.fom_func.__name__
        if not fom_func_name in fom_funcs.func_names:
            ShowWarningDialog(self.parent, 'The loaded fom function, ' \
                              +fom_func_name+', does not exist '+ \
                              'in the local fom_funcs file. The fom fucntion has been'+
                              ' temporary added to the list of availabe fom functions')
            fom_funcs.func_names.append(fom_func_name)
            exectext='fom_funcs.'+fom_func_name+ \
                     ' = self.parent.model.fom_func'
            exec(exectext, locals(), globals())

        dlg=SettingsDialog(frame, self.optimizer, self, fom_func_name)

        def applyfunc(object):
            self.WriteConfig()
            self.parent.model.set_fom_func(eval('fom_funcs.'+object.get_fom_string()))

        dlg.set_apply_change_func(applyfunc)

        dlg.ShowModal()
        # if dlg.ShowModal() == wx.ID_OK:
        #    pass
        dlg.Destroy()

    def TextOutput(self, text):
        '''TextOutput(self, text) --> None
        Function to present the output from the optimizer to the user. 
        Takes a string as input.
        '''
        # self.parent.main_frame_statusbar.SetStatusText(text, 0)
        evt=update_text(text=text)
        wx.PostEvent(self.parent, evt)
        # wx.CallAfter(wx.PostEvent, self.parent, evt)

    def PlotOutput(self, solver):
        ''' PlotOutput(self, solver) --> None
        Solver to present the graphical output from the optimizer to the 
        user. Takes the solver as input argument and picks out the 
        variables to show in the GUI.
        '''
        # print 'sending event plotting'
        # _post_solver_event(self.parent, solver, desc = 'Fitting update')
        evt=update_plot(model=solver.get_model(),
                        fom_log=solver.get_fom_log(), update_fit=solver.new_best,
                        desc='Fitting update')
        wx.PostEvent(self.parent, evt)
        # wx.CallAfter(wx.PostEvent, self.parent, evt)
        # Hard code the events for the plugins so that they can be run syncrously.
        # This is important since the Refelctevity model, for example, relies on the
        # current state of the model.
        try:
            self.parent.plugin_control.OnFittingUpdate(evt)
            # wx.CallAfter(self.parent.plugin_control.OnFittingUpdate, evt)
            # pass
        except Exception as e:
            iprint('Error in plot output:\n'+repr(e))

    def ParameterOutput(self, solver):
        '''ParameterOutput(self, solver) --> none
        
        Function to send an update event to update windows that displays
        the parameters to update the values. 
        Takes the solver as input argument and picks out the variables to 
        show in the GUI.
        '''
        evt=update_parameters(values=solver.best_vec.copy(),
                              new_best=solver.new_best,
                              population=solver.pop_vec,
                              max_val=solver.par_max,
                              min_val=solver.par_min,
                              fitting=True,
                              desc='Parameter Update', update_errors=False,
                              permanent_change=False)
        wx.PostEvent(self.parent, evt)
        # wx.CallAfter(wx.PostEvent, self.parent, evt)

    def ModelLoaded(self):
        '''ModelLoaded(self) --> None
        
        Function that takes care of resetting everything when a model has
        been loaded.
        '''
        evt=update_plot(model=self.optimizer.get_model(),
                        fom_log=self.optimizer.get_fom_log(), update_fit=False,
                        desc='Model loaded')
        wx.PostEvent(self.parent, evt)

        # Update the parameter plot ... 
        if self.optimizer.setup_ok:
            # remeber to add a check 
            solver=self.optimizer
            try:
                evt=update_parameters(values=solver.best_vec.copy(),
                                      new_best=False,
                                      population=solver.pop_vec,
                                      max_val=solver.par_max,
                                      min_val=solver.par_min,
                                      fitting=True,
                                      desc='Parameter Update', update_errors=False,
                                      permanent_change=False)
            except:
                iprint('Could not create data for paraemters')
            else:
                wx.PostEvent(self.parent, evt)

    def AutoSave(self):
        '''DoAutoSave(self) --> None
        
        Function that conducts an autosave of the model.
        '''
        io.save_file(self.parent.model.get_filename(), self.parent.model, self.optimizer, self.config)
        # print 'AutoSaved!'

    def FittingEnded(self, solver):
        '''FittingEnded(self, solver) --> None
        
        function used to post an event when the fitting has ended.
        This must be done since it is not htread safe otherwise. Same GUI in
        two threads when dialogs are run. dangerous...
        '''
        evt=fitting_ended(solver=solver, desc='Fitting Ended')
        wx.PostEvent(self.parent, evt)

    def OnFittingEnded(self, evt):
        '''OnFittingEnded(self, solver) --> None
        
        Callback when fitting has ended. Takes care of cleaning up after
        the fit. Calculates errors on the parameters and updates the grid.
        '''
        solver=evt.solver
        if solver.error:
            ShowErrorDialog(self.parent, solver.error)
            return

        message='Do you want to keep the parameter values from '+ \
                'the fit?'
        dlg=wx.MessageDialog(self.parent, message, 'Keep the fit?',
                             wx.YES_NO | wx.ICON_QUESTION)
        if dlg.ShowModal()==wx.ID_YES:
            evt=update_parameters(values=solver.best_vec.copy(),
                                  desc='Parameter Update', new_best=True,
                                  update_errors=False, fitting=False,
                                  permanent_change=True)
            wx.PostEvent(self.parent, evt)
        else:
            # print 'Resetting the values in the grid to ',\
            #    self.start_parameter_values
            evt=update_parameters(values=solver.start_guess,
                                  desc='Parameter Update', new_best=True,
                                  update_errors=False, fitting=False,
                                  permanent_change=False)
            wx.PostEvent(self.parent, evt)

    def CalcErrorBars(self):
        '''
        Method that calculates the errorbars for the fit that has been
        done. Note that the fit has to been conducted before this is run.
        '''
        if len(self.optimizer.fom_evals)==0:
            raise ErrorBarError('Can not find any stored evaluations of the model in the optimizer.\n'
                                'Run a fit before calculating the errorbars.')
        if self.optimizer.start_guess is not None and not self.optimizer.running:
            n_elements=len(self.optimizer.start_guess)
            # print 'Number of elemets to calc errobars for ', n_elements
            error_values=[]
            dlg=wx.ProgressDialog("Calculating", "Error bars are calculated ...",
                                  maximum=n_elements, parent=self.parent,
                                  style=wx.PD_AUTO_HIDE)
            for index in range(n_elements):
                # calculate the error
                # TODO: Check the error bar buisness again and how to treat 
                # Chi2 
                try:
                    (error_low, error_high)=self.optimizer.calc_error_bar(index, self.fom_error_bars_level)
                except diffev.ErrorBarError as e:
                    ShowWarningDialog(self.parent, str(e))
                    break
                error_str='(%.3e, %.3e)'%(error_low, error_high)
                error_values.append(error_str)
                dlg.Update(index+1)

            dlg.Destroy()
            return error_values
        else:
            raise ErrorBarError()

    def ProjectEvals(self, parameter):
        ''' ProjectEvals(self, parameter) --> prameter, fomvals
        
        Projects the parameter number parameter on one axis and returns
        the fomvals.
        '''
        model=self.parent.model
        row=model.parameters.get_pos_from_row(parameter)
        if self.optimizer.start_guess is not None and not self.optimizer.running:
            return self.optimizer.par_evals[:, row], \
                   self.optimizer.fom_evals[:]
        else:
            raise ErrorBarError()

    def ScanParameter(self, parameter, points):
        '''ScanParameter(self, parameter, points) 
            --> par_vals, fom_vals
        
        Scans one parameter and records its fom value as a function 
        of the parameter value.
        '''
        row=parameter
        model=self.parent.model
        (funcs, vals)=model.get_sim_pars()
        minval=model.parameters.get_data()[row][3]
        maxval=model.parameters.get_data()[row][4]
        parfunc=funcs[model.parameters.get_sim_pos_from_row(row)]
        par_def_val=vals[model.parameters.get_sim_pos_from_row(row)]
        step=(maxval-minval)/points
        par_vals=np.arange(minval, maxval+step, step)
        fom_vals=np.array([])

        par_name=model.parameters.get_data()[row][0]
        dlg=wx.ProgressDialog("Scan Parameter",
                              "Scanning parameter "+par_name,
                              maximum=len(par_vals),
                              parent=self.parent,
                              style=wx.PD_APP_MODAL | wx.PD_ELAPSED_TIME
                                    | wx.PD_REMAINING_TIME | wx.PD_AUTO_HIDE)
        try:
            # Start with setting all values
            [f(v) for (f, v) in zip(funcs, vals)]
            for par_val in par_vals:
                parfunc(par_val)
                fom_vals=np.append(fom_vals, model.evaluate_fit_func())
                dlg.Update(len(fom_vals))
        except Exception as e:
            dlg.Destroy()
            outp=io.StringIO()
            traceback.print_exc(200, outp)
            val=outp.getvalue()
            outp.close()
            ShowWarningDialog(self.parent, 'Error while evaluatating the'+ \
                              ' simulation and fom. Please check so it is possible to simulate'+ \
                              ' your model. Detailed output below: \n\n'+val)
        else:
            dlg.Destroy()
        # resetting the scanned parameter
        parfunc(par_def_val)
        return par_vals, fom_vals

    def ResetOptimizer(self):
        '''ResetOptimizer(self) --> None
        
        Resets the optimizer - clears the memory and special flags.
        '''
        self.start_parameter_values=None

    def StartFit(self):
        ''' StartFit(self) --> None
        Function to start running the fit
        '''
        # Make sure that the config of the solver is updated..
        self.ReadConfig()
        model=self.parent.model
        # Reset all the errorbars
        model.parameters.clear_error_pars()
        # self.start_parameter_values = model.get_fit_values()
        self.optimizer.start_fit(model)
        # print 'Optimizer starting'

    def StopFit(self):
        ''' StopFit(self) --> None
        Function to stop a running fit
        '''
        self.optimizer.stop_fit()

    def ResumeFit(self):
        ''' ResumeFit(self) --> None
        
        Function to resume the fitting after it has been stopped
        '''
        # Make sure the settings are updated..
        self.ReadConfig()
        model=self.parent.model
        # Remove all previous erros ...

        self.optimizer.resume_fit(model)

    def IsFitted(self):
        '''IsFitted(self) --> bool
        
        Returns true if a fit has been started otherwise False
        '''
        return len(self.optimizer.start_guess)>0

    def set_error_bars_level(self, value):
        '''set_error_bars_level(value) --> None
        
        Sets the value of increase of the fom used for errorbar calculations
        '''
        if value<1:
            raise ValueError('fom_error_bars_level has to be above 1')
        else:
            self.fom_error_bars_level=value

    def set_save_all_evals(self, value):
        '''Sets the boolean value to save all evals to file
        '''
        self.save_all_evals=bool(value)

# ==============================================================================
class SettingsDialog(wx.Dialog):
    def __init__(self, parent, solver, solvergui, fom_string):
        '''__init__(self, parent, solver, fom_string, mut_schemes,\
                    current_mut_scheme)
                    
        parent - parent window, solver - the solver (Diffev alg.)
        fom_string - the fom function string
        '''
        wx.Dialog.__init__(self, parent, -1, 'Optimizer settings')
        # self.SetAutoLayout(True)
        self.solver=solver
        self.solvergui=solvergui
        self.apply_change=None

        col_sizer=wx.BoxSizer(wx.HORIZONTAL)
        row_sizer1=wx.BoxSizer(wx.VERTICAL)

        # FOM BOX SIZER
        fom_box=wx.StaticBox(self, -1, "FOM")
        fom_box_sizer=wx.StaticBoxSizer(fom_box, wx.VERTICAL)

        # FOM choice
        fom_sizer=wx.BoxSizer(wx.HORIZONTAL)
        fom_text=wx.StaticText(self, -1, 'Figure of merit ')
        self.fom_choice=wx.Choice(self, -1, choices=fom_funcs.func_names)
        self.fom_choice.SetSelection(fom_funcs.func_names.index(fom_string))
        fom_sizer.Add(fom_text, 0,
                      wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, border=10)
        fom_sizer.Add(self.fom_choice, 0, wx.EXPAND, border=10)
        fom_box_sizer.Add(fom_sizer, 0, wx.ALIGN_CENTRE | wx.ALL, 5)

        cb_sizer=wx.BoxSizer()
        fom_box_sizer.Add(cb_sizer, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        # Check box for ignoring nans
        self.fom_ignore_nan_control=wx.CheckBox(self, -1, "Ignore Nan")
        cb_sizer.Add(self.fom_ignore_nan_control, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        self.fom_ignore_nan_control.SetValue(self.solvergui.parent.model.fom_ignore_nan)
        # Check box for ignoring infs
        self.fom_ignore_inf_control=wx.CheckBox(self, -1, "Ignore +/-Inf")
        cb_sizer.Add(self.fom_ignore_inf_control, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        self.fom_ignore_inf_control.SetValue(self.solvergui.parent.model.fom_ignore_inf)
        self.fom_ignore_inf_control.SetValue(self.solvergui.parent.model.fom_ignore_inf)

        # Errorbar level 
        errorbar_sizer=wx.BoxSizer(wx.HORIZONTAL)
        errorbar_text=wx.StaticText(self, -1, 'Error bar level ')
        self.errorbar_control=NumCtrl(self, value=
        self.solvergui.fom_error_bars_level,
                                      fractionWidth=2, integerWidth=2)
        errorbar_sizer.Add(errorbar_text, 0,
                           wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, border=10)
        errorbar_sizer.Add(self.errorbar_control, 1, wx.ALIGN_CENTER_VERTICAL, border=10)
        errorbar_sizer.Add((10, 20), 0, wx.EXPAND)
        fom_box_sizer.Add(errorbar_sizer, 0, wx.ALIGN_CENTRE | wx.ALL, 5)

        # X-Range limiting
        cb_sizer=wx.BoxSizer(wx.HORIZONTAL)
        fom_box_sizer.Add(cb_sizer, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        self.limit_fit_range=wx.CheckBox(self, -1, "Limit x-range")
        cb_sizer.Add(self.limit_fit_range, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        self.limit_fit_range.SetValue(self.solver.limit_fit_range)

        cb_sizer=wx.BoxSizer(wx.HORIZONTAL)
        fom_box_sizer.Add(cb_sizer, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        cb_sizer.Add(wx.StaticText(self, -1, 'x_min'))
        self.fit_xmin=wx.SpinCtrlDouble(self, -1, min=0., max=99.0,
                                        initial=self.solver.fit_xmin, inc=0.01)
        cb_sizer.Add(self.fit_xmin)
        cb_sizer.Add(wx.StaticText(self, -1, 'x_max'))
        self.fit_xmax=wx.SpinCtrlDouble(self, -1, min=0., max=99.0,
                                        initial=self.solver.fit_xmax, inc=0.01)
        cb_sizer.Add(self.fit_xmax)

        row_sizer1.Add(fom_box_sizer, 0, wx.EXPAND, 5)

        # Make the Fitting box
        fit_box=wx.StaticBox(self, -1, "Fitting")
        fit_box_sizer=wx.StaticBoxSizer(fit_box, wx.VERTICAL)

        # Make a sizer for the check boxes
        cb_sizer=wx.BoxSizer(wx.HORIZONTAL)
        fit_box_sizer.Add(cb_sizer, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        # Check box for start guess
        startguess_control=wx.CheckBox(self, -1, "Start guess")
        cb_sizer.Add(startguess_control, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        startguess_control.SetValue(self.solver.use_start_guess)
        self.startguess_control=startguess_control

        # Check box for using boundaries
        bound_control=wx.CheckBox(self, -1, "Use (Max, Min)")
        cb_sizer.Add(bound_control, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        bound_control.SetValue(self.solver.use_boundaries)
        self.bound_control=bound_control

        # Check box and integer input for autosave
        autosave_sizer=wx.BoxSizer(wx.HORIZONTAL)
        use_autosave_control=wx.CheckBox(self, -1, "Autosave, interval ")
        use_autosave_control.SetValue(self.solver.use_autosave)
        autosave_sc=wx.SpinCtrl(self)
        autosave_sc.SetRange(1, 1000)
        autosave_sc.SetValue(self.solver.autosave_interval)
        autosave_sc.Enable(True)
        autosave_sizer.Add(use_autosave_control, 0,
                           wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, border=5)
        autosave_sizer.Add(autosave_sc, 0, wx.ALIGN_CENTER_VERTICAL, border=5)
        self.autosave_sc=autosave_sc
        self.use_autosave_control=use_autosave_control
        fit_box_sizer.Add(autosave_sizer, 0, wx.ALIGN_CENTRE | wx.ALL, 5)

        # Checkbox for saving all evals
        save_sizer=wx.BoxSizer(wx.HORIZONTAL)
        save_all_control=wx.CheckBox(self, -1, "Save evals, buffer ")
        save_all_control.SetValue(self.solvergui.save_all_evals)
        buffer_sc=wx.SpinCtrl(self)
        buffer_sc.SetRange(1000, 100000000)
        buffer_sc.SetValue(self.solver.max_log)
        buffer_sc.Enable(True)
        save_sizer.Add(save_all_control, 0,
                       wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, border=5)
        save_sizer.Add(buffer_sc, 0, wx.ALIGN_CENTER_VERTICAL, border=5)
        self.buffer_sc=buffer_sc
        self.save_all_control=save_all_control
        fit_box_sizer.Add(save_sizer, 0, wx.ALIGN_CENTRE | wx.ALL, 5)

        row_sizer1.Add(fit_box_sizer, 1, wx.EXPAND, 5)

        col_sizer.Add(row_sizer1, 1, wx.ALIGN_CENTRE | wx.ALL, 5)

        row_sizer2=wx.BoxSizer(wx.VERTICAL)

        # Make the Diff. Ev. box
        de_box=wx.StaticBox(self, -1, "Diff. Ev.")
        de_box_sizer=wx.StaticBoxSizer(de_box, wx.VERTICAL)
        de_grid=wx.GridBagSizer(2, 2)

        km_sizer=wx.BoxSizer(wx.HORIZONTAL)
        km_text=wx.StaticText(self, -1, 'k_m ')
        self.km_control=NumCtrl(self, value=self.solver.km,
                                fractionWidth=2, integerWidth=2)
        km_sizer.Add(km_text, 0,
                     wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, border=10)
        km_sizer.Add(self.km_control, 1, wx.ALIGN_CENTER_VERTICAL, border=10)
        km_sizer.Add((10, 20), 0, wx.EXPAND)
        de_grid.Add(km_sizer, (0, 0),
                    flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL,
                    border=5)

        kr_sizer=wx.BoxSizer(wx.HORIZONTAL)
        kr_sizer.Add((10, 20), 0, wx.EXPAND)
        kr_text=wx.StaticText(self, -1, 'k_r ')
        self.kr_control=NumCtrl(self, value=self.solver.kr,
                                fractionWidth=2, integerWidth=2)
        kr_sizer.Add(kr_text, 1,
                     wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, border=10)
        kr_sizer.Add(self.kr_control, 0, wx.ALIGN_CENTER_VERTICAL, border=10)
        de_grid.Add(kr_sizer, (0, 1),
                    flag=wx.ALIGN_RIGHT | wx.ALIGN_CENTER_VERTICAL,
                    border=5)

        method_sizer=wx.BoxSizer(wx.HORIZONTAL)
        method_text=wx.StaticText(self, -1, 'Method ')
        mut_schemes=[f.__name__ for f in self.solver.mutation_schemes]
        self.method_choice=wx.Choice(self, -1,
                                     choices=mut_schemes)
        self.method_choice.SetSelection(self.solver.get_create_trial(True))
        method_sizer.Add(method_text, 0,
                         wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, border=10)
        method_sizer.Add(self.method_choice, 0, wx.ALIGN_CENTER_VERTICAL, border=10)
        de_grid.Add(method_sizer, (1, 0), (1, 2),
                    flag=wx.ALIGN_CENTER | wx.ALIGN_CENTER_VERTICAL | wx.EXPAND,
                    border=5)

        de_box_sizer.Add(de_grid, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        row_sizer2.Add(de_box_sizer, 0, wx.EXPAND, 5)

        # Make the Population box
        pop_box=wx.StaticBox(self, -1, "Population size")
        pop_box_sizer=wx.StaticBoxSizer(pop_box, wx.VERTICAL)
        pop_grid=wx.FlexGridSizer(0, 2, 0, 0)

        multsize_radio=wx.RadioButton(self, -1, " Relative size ",
                                      style=wx.RB_GROUP)
        fixedsize_radio=wx.RadioButton(self, -1, " Fixed size ")

        multsize_sc=wx.SpinCtrl(self)
        multsize_sc.SetRange(1, 1000)
        multsize_sc.SetValue(self.solver.pop_mult)
        multsize_sc.Enable(self.solver.use_pop_mult)
        fixedsize_sc=wx.SpinCtrl(self)
        fixedsize_sc.SetRange(1, 1000)
        fixedsize_sc.SetValue(self.solver.pop_size)
        fixedsize_sc.Enable(not self.solver.use_pop_mult)

        self.pop_multsize_radio=multsize_radio
        self.pop_fixedsize_radio=fixedsize_radio
        self.pop_multsize_sc=multsize_sc
        self.pop_fixedsize_sc=fixedsize_sc

        pop_grid.Add(multsize_radio, 0,
                     wx.ALIGN_LEFT | wx.LEFT | wx.RIGHT | wx.TOP, 5)
        pop_grid.Add(multsize_sc, 0,
                     wx.ALIGN_RIGHT | wx.LEFT | wx.RIGHT | wx.TOP, 5)
        pop_grid.Add(fixedsize_radio, 0,
                     wx.ALIGN_LEFT | wx.LEFT | wx.RIGHT | wx.TOP, 5)
        pop_grid.Add(fixedsize_sc, 0,
                     wx.ALIGN_RIGHT | wx.LEFT | wx.RIGHT | wx.TOP, 5)

        pop_box_sizer.Add(pop_grid, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        row_sizer2.Add(pop_box_sizer, 1, wx.EXPAND, 5)
        self.Bind(wx.EVT_RADIOBUTTON, self.on_pop_select, multsize_radio)
        self.Bind(wx.EVT_RADIOBUTTON, self.on_pop_select, fixedsize_radio)
        multsize_radio.SetValue(self.solver.use_pop_mult)
        fixedsize_radio.SetValue(not self.solver.use_pop_mult)

        # Make the Generation box
        gen_box=wx.StaticBox(self, -1, "Max Generations")
        gen_box_sizer=wx.StaticBoxSizer(gen_box, wx.VERTICAL)
        gen_grid=wx.FlexGridSizer(0, 2, 0, 0)

        gen_multsize_radio=wx.RadioButton(self, -1, " Relative size ",
                                          style=wx.RB_GROUP)
        gen_fixedsize_radio=wx.RadioButton(self, -1, " Fixed size ")

        gen_multsize_sc=wx.SpinCtrl(self)
        gen_multsize_sc.SetRange(1, 10000)
        gen_multsize_sc.SetValue(self.solver.max_generation_mult)
        gen_multsize_sc.Enable(not self.solver.use_max_generations)
        gen_fixedsize_sc=wx.SpinCtrl(self)
        gen_fixedsize_sc.SetRange(1, 10000)
        gen_fixedsize_sc.SetValue(self.solver.max_generations)
        gen_fixedsize_sc.Enable(self.solver.use_max_generations)

        self.gen_multsize_radio=gen_multsize_radio
        self.gen_fixedsize_radio=gen_fixedsize_radio
        self.gen_multsize_sc=gen_multsize_sc
        self.gen_fixedsize_sc=gen_fixedsize_sc

        gen_grid.Add(gen_multsize_radio, 0,
                     wx.ALIGN_LEFT | wx.LEFT | wx.RIGHT | wx.TOP, 5)
        gen_grid.Add(gen_multsize_sc, 0,
                     wx.ALIGN_CENTRE | wx.LEFT | wx.RIGHT | wx.TOP, 5)
        gen_grid.Add(gen_fixedsize_radio, 0,
                     wx.ALIGN_LEFT | wx.LEFT | wx.RIGHT | wx.TOP, 5)
        gen_grid.Add(gen_fixedsize_sc, 0,
                     wx.ALIGN_CENTRE | wx.LEFT | wx.RIGHT | wx.TOP, 5)

        gen_box_sizer.Add(gen_grid, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        row_sizer2.Add(gen_box_sizer, 1, wx.EXPAND, 5)
        self.Bind(wx.EVT_RADIOBUTTON, self.on_gen_select, gen_multsize_radio)
        self.Bind(wx.EVT_RADIOBUTTON, self.on_gen_select, gen_fixedsize_radio)
        gen_fixedsize_radio.SetValue(self.solver.use_max_generations)
        gen_multsize_radio.SetValue(not self.solver.use_max_generations)

        ##
        # Make the parallel fitting box
        parallel_box=wx.StaticBox(self, -1, "Parallel processing")
        parallel_box_sizer=wx.StaticBoxSizer(parallel_box, wx.VERTICAL)

        use_parallel_control=wx.CheckBox(self, -1, "Parallel fitting")
        use_parallel_control.SetValue(self.solver.use_parallel_processing)
        use_parallel_control.Enable(diffev.__parallel_loaded__)
        self.use_parallel_control=use_parallel_control
        parallel_box_sizer.Add(use_parallel_control, 1, wx.EXPAND, 5)

        processes_sc=wx.SpinCtrl(self, size=(80, -1))
        processes_sc.SetRange(1, 100)
        processes_sc.SetValue(self.solver.processes)
        processes_sc.Enable(diffev.__parallel_loaded__)
        chunk_size_sc=wx.SpinCtrl(self, size=(80, -1))
        chunk_size_sc.SetRange(1, 100)
        chunk_size_sc.SetValue(self.solver.chunksize)
        chunk_size_sc.Enable(diffev.__parallel_loaded__)
        self.processes_sc=processes_sc
        self.chunk_size_sc=chunk_size_sc
        parallel_sizer=wx.BoxSizer(wx.HORIZONTAL)
        p_text=wx.StaticText(self, -1, '# Processes')
        parallel_sizer.Add(p_text, 0,
                           wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, border=10)
        parallel_sizer.Add((10, 20), 1, wx.EXPAND)
        parallel_sizer.Add(processes_sc, 0, wx.ALIGN_CENTER_VERTICAL, border=10)
        parallel_box_sizer.Add(parallel_sizer, 1, wx.EXPAND, 10)
        parallel_sizer=wx.BoxSizer(wx.HORIZONTAL)
        p_text=wx.StaticText(self, -1, ' Chunk size ')
        parallel_sizer.Add(p_text, 0,
                           wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, border=10)
        parallel_sizer.Add((10, 20), 1, wx.EXPAND)
        parallel_sizer.Add(chunk_size_sc, 0, wx.ALIGN_CENTER_VERTICAL, border=10)

        parallel_box_sizer.Add(parallel_sizer, 1, wx.EXPAND, 10)
        row_sizer2.Add(parallel_box_sizer, 1, wx.EXPAND, 5)

        col_sizer.Add(row_sizer2, 1, wx.ALIGN_CENTRE | wx.ALL, 5)
        ##

        # Add the Dialog buttons
        button_sizer=wx.StdDialogButtonSizer()
        okay_button=wx.Button(self, wx.ID_OK)
        okay_button.SetDefault()
        button_sizer.AddButton(okay_button)
        apply_button=wx.Button(self, wx.ID_APPLY)
        apply_button.SetDefault()
        button_sizer.AddButton(apply_button)
        button_sizer.AddButton(wx.Button(self, wx.ID_CANCEL))
        button_sizer.Realize()
        # Add some eventhandlers
        self.Bind(wx.EVT_BUTTON, self.on_apply_change, okay_button)
        self.Bind(wx.EVT_BUTTON, self.on_apply_change, apply_button)

        sizer=wx.BoxSizer(wx.VERTICAL)
        sizer.Add(col_sizer, 1, wx.GROW, 20)
        # sizer.Add(col_sizer, 1, wx.GROW|wx.ALL|wx.EXPAND, 20)
        line=wx.StaticLine(self, -1, size=(20, -1), style=wx.LI_HORIZONTAL)
        sizer.Add(line, 0, wx.GROW | wx.TOP, 20)

        sizer.Add(button_sizer, 0,
                  flag=wx.ALIGN_RIGHT, border=20)
        self.SetSizer(sizer)

        sizer.Fit(self)
        self.Layout()

    def on_pop_select(self, event):
        '''on_pop_select(self, event) --> None
        
        callback for selction of a radio button in the population group
        '''
        radio_selected=event.GetEventObject()

        if radio_selected is self.pop_fixedsize_radio:
            self.pop_fixedsize_sc.Enable(True)
            self.pop_multsize_sc.Enable(False)
        else:
            self.pop_fixedsize_sc.Enable(False)
            self.pop_multsize_sc.Enable(True)

    def on_gen_select(self, event):
        '''on_pop_select(self, event) --> None
        
        callback for selection of a radio button in the Generation group
        '''
        radio_selected=event.GetEventObject()

        if radio_selected is self.gen_fixedsize_radio:
            self.gen_fixedsize_sc.Enable(True)
            self.gen_multsize_sc.Enable(False)
        else:
            self.gen_fixedsize_sc.Enable(False)
            self.gen_multsize_sc.Enable(True)

    def get_fom_string(self):
        return self.fom_choice.GetStringSelection()

    def on_apply_change(self, event):
        self.solver.kr=self.kr_control.GetValue()
        self.solver.km=self.km_control.GetValue()
        self.solver.max_generation_mult=self.gen_multsize_sc.GetValue()
        self.solver.max_generations=self.gen_fixedsize_sc.GetValue()
        self.solver.pop_mult=self.pop_multsize_sc.GetValue()
        self.solver.pop_size=self.pop_fixedsize_sc.GetValue()
        self.solver.use_max_generations=self.gen_fixedsize_radio.GetValue()
        self.solver.use_pop_mult=self.pop_multsize_radio.GetValue()
        self.solver.use_start_guess=self.startguess_control.GetValue()
        self.solver.use_boundaries=self.bound_control.GetValue()
        self.solver.set_create_trial(self.method_choice.GetStringSelection())
        self.solver.use_parallel_processing=self.use_parallel_control.GetValue()
        self.solver.processes=self.processes_sc.GetValue()
        self.solver.chunksize=self.chunk_size_sc.GetValue()
        self.solvergui.parent.model.set_fom_ignore_inf(self.fom_ignore_inf_control.GetValue())
        self.solvergui.parent.model.set_fom_ignore_nan(self.fom_ignore_nan_control.GetValue())
        self.solvergui.fom_error_bars_level=self.errorbar_control.GetValue()
        self.solver.use_autosave=self.use_autosave_control.GetValue()
        self.solver.autosave_interval=self.autosave_sc.GetValue()
        self.solvergui.save_all_evals=self.save_all_control.GetValue()
        self.solver.max_log=self.buffer_sc.GetValue()
        self.solver.limit_fit_range=self.limit_fit_range.GetValue()
        self.solver.fit_xmin=self.fit_xmin.GetValue()
        self.solver.fit_xmax=self.fit_xmax.GetValue()

        model=self.solvergui.parent.model
        model.limit_fit_range, model.fit_xmin, model.fit_xmax=(
            self.solver.limit_fit_range,
            self.solver.fit_xmin,
            self.solver.fit_xmax)

        if self.apply_change:
            self.apply_change(self)

        event.Skip()

    def set_apply_change_func(self, func):
        '''set_apply_change_func(self, func) --> None
        
        Set the apply_change function. Is executed when the apply or ok button
        is clicked.
        '''
        self.apply_change=func

# ==============================================================================
# Custom events needed for updating and message parsing between the different
# modules.

(update_plot, EVT_UPDATE_PLOT)=wx.lib.newevent.NewEvent()
(update_text, EVT_SOLVER_UPDATE_TEXT)=wx.lib.newevent.NewEvent()
(update_parameters, EVT_UPDATE_PARAMETERS)=wx.lib.newevent.NewEvent()
(fitting_ended, EVT_FITTING_ENDED)=wx.lib.newevent.NewEvent()

# ==============================================================================
def ShowWarningDialog(frame, message):
    dlg=wx.MessageDialog(frame, message,
                         'Warning',
                         wx.OK | wx.ICON_WARNING
                         )
    dlg.ShowModal()
    dlg.Destroy()

def ShowErrorDialog(frame, message, position=''):
    if position!='':
        dlg=wx.MessageDialog(frame, message+'\n'+'Position: '+position,
                             'ERROR',
                             wx.OK | wx.ICON_ERROR
                             )
    else:
        dlg=wx.MessageDialog(frame, message,
                             'ERROR',
                             wx.OK | wx.ICON_ERROR
                             )
    dlg.ShowModal()
    dlg.Destroy()

class GenericError(Exception):
    ''' Just a empty class used for inheritance. Only useful
    to check if the errors are originating from the model library.
    All these errors are controllable. If they not originate from
    this class something has passed trough and that should be impossible '''
    pass

class ErrorBarError(GenericError):
    def __init__(self, error_message=None):
        """Error class for the fom evaluation

        :param error_message: Error message that explains the error, string.
        :return:
        """
        if error_message is None:
            self.error_message='Could not evaluate the error bars. A fit has to be run before they can be calculated'
        else:
            self.error_message=error_message

    def __str__(self):
        return self.error_message



'''data_loader_framework.py

Library that implements a template (Template) class for classes that
loads data into GenX.
'''

class Template:
    wildcard=None

    def __init__(self, parent=None):
        self.parent=parent

        if parent is not None:
            # This is made for the virtual datalist controller...
            self.data=self.parent.data_cont.get_data()
            self.Register()

    def Register(self):
        '''Register(self) --> None
        
        Register the data loader with the parent
        '''
        self.parent.data_loader=self

    def SetData(self, data):
        '''SetData(self, data) --> None
        
        Sets the data connection to the plugin.
        '''
        self.data=data

    def UpdateDataList(self):
        '''UpdateDataList(self) --> None
        
        Forces the data list to update. This is only necessary if new
        data sets have been added when the data has been loaded
        '''
        # Just a force update of the data_list
        self.parent.SetItemCount(
            self.parent.data_cont.get_count())
        # Updating the imagelist as well
        self.parent._UpdateImageList()

    def SetStatusText(self, text):
        '''SetStatusText(self, text) --> None
        
        Set a status text in the main frame for user information 
        '''
        self.parent.SetStatusText(text)

    def LoadDataFile(self, selected_items):
        '''LoadDataFile(self, seleceted_items) --> None
        
        Selected items is the selcted items in the items in the current DataList
        into which data from file(s) should be loaded. Note that the default
        implementation only allows the loading of a single file! 
        Overriding this function in subclasses can of course change this 
        behaviour. This function calls the LoadData function which implements 
        the io function by it self. The LoadData has to be overloaded in
        order to have a working plugin.
        '''
        import wx
        n_selected=len(selected_items)
        if n_selected==1:
            if self.wildcard is None:
                wc="All files (*.*)|*.*"
            else:
                wc="%s (%s)|%s|All files (*.*)|*.*"%(self.__module__.rsplit('.', 1)[1].capitalize(),
                                                     self.wildcard, self.wildcard)
            dlg=wx.FileDialog(self.parent, message="Choose your Datafile"
                              , defaultFile="", wildcard=wc
                              , style=wx.FD_OPEN | wx.FD_CHANGE_DIR)

            if dlg.ShowModal()==wx.ID_OK:
                self.SetData(self.parent.data_cont.get_data())
                dataset=self.data[selected_items[0]]
                # in case the data loader does not define any metadata
                # at least set the instrument to data loader name
                dataset.meta['data_source']['experiment']['instrument']=self.__module__.rsplit('.', 1)[1]
                self.LoadData(dataset, dlg.GetPath())
                dataset.meta['data_source']['file_name']=dlg.GetPath()

                # In case the dataset name has changed
                self.UpdateDataList()

                # Send an update that new data has been loaded
                self.SendUpdateDataEvent()

                return True
            dlg.Destroy()
        else:
            if n_selected>1:
                dlg=wx.MessageDialog(self.parent,
                                     'Please select only one dataset'
                                     , caption='Too many selections'
                                     , style=wx.OK | wx.ICON_INFORMATION)
            else:
                dlg=wx.MessageDialog(self.parent, 'Please select a dataset'
                                     , caption='No active dataset'
                                     , style=wx.OK | wx.ICON_INFORMATION)
            dlg.ShowModal()
            dlg.Destroy()

        return False

    def LoadData(self, dataset, file_path):
        '''LoadData(self, dataset, file_path) --> None
        
        This file should load a single data file into data object of
        the model. Please overide this function. It is called by the 
        LoadFile function.
        '''
        pass

    def SettingsDialog(self):
        '''SettingsDialog(self) --> None
        
        This function should - if necessary implement a dialog box
        that allows the user set import settings for example.
        '''
        pass

    def SendUpdateDataEvent(self):
        '''SendUpdateDataEvent(self) --> None
        
        Sends an event that new data has been loaded and 
        things such as plotting should be updated.
        '''
        self.parent._UpdateData('New data added')

    def Remove(self):
        '''Remove(self) --> None
        
        Removes the plugin from knowledge of the parent.
        '''
        self.parent.data_loader=None



'''
Library for the GUI components of the Parameter grid which is used to 
define which parameters to fit. The library parameters contains the class
definition of the parameters. 
Programmer: Matts Bjorck
Last Changed: 2014 07 17
'''

import os
import wx
import wx.grid as gridlib
import wx.lib.printout as printout
import wx.lib.agw.floatspin as fs
from wx.lib.masked import NumCtrl
import string

from numpy import *

from . import parameters
from . import images as img
from .lib import controls as ctrls
from . import filehandling
from .gui_logging import iprint

# =============================================================================
# class ParameterDataTable
class ParameterDataTable(gridlib.GridTableBase):
    '''
    Class for the datatable which is used by the grid. 
    Mostly a compatability layer between the Parameters class
    and the grid.
    '''

    def __init__(self, parent):
        gridlib.GridTableBase.__init__(self)
        self.parent=parent
        self.pars=parameters.Parameters()

        self.data_types=[gridlib.GRID_VALUE_STRING,
                         gridlib.GRID_VALUE_FLOAT,
                         gridlib.GRID_VALUE_BOOL,
                         gridlib.GRID_VALUE_FLOAT,
                         gridlib.GRID_VALUE_FLOAT,
                         gridlib.GRID_VALUE_STRING,
                         ]

    # required methods for the wxGridTableBase interface

    def GetNumberRows(self):
        return self.pars.get_len_rows()

    def GetNumberCols(self):
        return self.pars.get_len_cols()

    def GetRowLabelValue(self, row):
        if row<self.pars.get_len_rows() and self.pars.get_value(row, 2):
            number=sum([self.pars.get_value(i, 2) for i in range(row)])
            return "%d"%int(number)
        else:
            return '-'

    def IsEmptyCell(self, row, col):
        try:
            return not self.pars.get_value(row, col)
        except IndexError:
            return True

    def GetValue(self, row, col):
        try:
            return self.pars.get_value(row, col)
        except IndexError:
            return ''

    def SetValue(self, row, col, value):
        try:
            self.pars.set_value(row, col, value)
        except IndexError as e:
            # add a new row
            self.pars.append()
            # print 'Value:', value

            # self.SetValue(row, col, value)
            # tell the grid we've added a row
            msg=gridlib.GridTableMessage(self,  # The table
                                         gridlib.GRIDTABLE_NOTIFY_ROWS_APPENDED,  # what we did to it
                                         1  # how many
                                         )

            self.GetView().ProcessTableMessage(msg)
            self.pars.set_value(row, col, value)
        # For updating the column labels according to the number of fitted parameters
        if col==2 or col==3 or col==4:
            self.GetView().ForceRefresh()
        self.parent._grid_changed()

    def DeleteRows(self, rows):
        delete_count=self.pars.delete_rows(rows)

        msg=gridlib.GridTableMessage(self,
                                     gridlib.GRIDTABLE_NOTIFY_ROWS_DELETED, self.GetNumberRows(),
                                     delete_count)
        self.GetView().ProcessTableMessage(msg)
        self.parent._grid_changed()
        # print self.data

    def InsertRow(self, row):
        self.pars.insert_row(row)

        msg=gridlib.GridTableMessage(self,
                                     gridlib.GRIDTABLE_NOTIFY_ROWS_APPENDED, 1)
        self.GetView().ProcessTableMessage(msg)
        self.GetView().ForceRefresh()
        self.parent._grid_changed()
        return True

    def MoveRowUp(self, row):
        """ Move row up one row.

        :param row: Integer row number to move up
        :return: Boolean
        """

        success=self.pars.move_row_up(row)

        self.GetView().ForceRefresh()
        self.parent._grid_changed()
        return success

    def MoveRowDown(self, row):
        """ Move row down one row.

        :param row: Integer row number to move down
        :return: Boolean
        """

        success=self.pars.move_row_down(row)

        self.GetView().ForceRefresh()
        self.parent._grid_changed()
        return success

    def SortRows(self):
        """ Sort the rows in the table

        :return: Boolean to indicate success
        """
        success=self.pars.sort_rows()

        self.GetView().ForceRefresh()
        self.parent._grid_changed()
        return success

    def AppendRows(self, num_rows=1):
        # print num_rows
        [self.pars.append() for i in range(num_rows)]

        msg=gridlib.GridTableMessage(self,
                                     gridlib.GRIDTABLE_NOTIFY_ROWS_APPENDED, num_rows)
        self.GetView().ProcessTableMessage(msg)
        self.GetView().ForceRefresh()
        self.parent._grid_changed()
        return True

    # def GetAttr(self, row, col, kind):
    #    '''Called by the grid to find the attributes of the cell,
    #    bkg color, text colour, font and so on.
    #    '''
    #    attr = super(ParameterDataTable, self).GetAttr(row, col, kind)

    #    if col == 1 and row < self.pars.get_len_rows():
    #        if attr is None:
    #            attr = gridlib.GridCellAttr()
    #        attr = attr.Clone()
    #        val = self.pars.get_value(row,1)
    #        max_val = self.pars.get_value(row,4)
    #        min_val = self.pars.get_value(row,3)
    #        if val > max_val or val < min_val:
    #            attr.SetBackgroundColour(wx.Colour(204, 0, 0))
    #            attr.SetTextColour(wx.Colour(255, 255, 255))

    #    return attr

    def GetColLabelValue(self, col):
        '''Called when the grid needs to display labels
        '''
        return self.pars.get_col_headers()[col]

    def GetTypeName(self, row, col):
        '''Called to determine the kind of editor/renderer to use by
        default, doesn't necessarily have to be the same type used
        natively by the editor/renderer if they know how to convert.
        '''
        return self.data_types[col]

    def CanGetValueAs(self, row, col, type_name):
        '''Called to determine how the data can be fetched and stored by the
        editor and renderer.  This allows you to enforce some type-safety
        in the grid.
        '''
        col_type=self.data_types[col].split(':')[0]
        if type_name==col_type:
            return True
        else:
            return False

    def CanSetValueAs(self, row, col, type_name):
        return self.CanGetValueAs(row, col, type_name)

    def SetParameters(self, pars, clear=True, permanent_change=True):
        '''
        SetParameters(self, pars) --> None
        
        Set the parameters in the table to pars. 
        pars has to an instance of Parameters.
        '''
        if clear:
            # Start by deleting all rows:
            msg=gridlib.GridTableMessage(self,
                                         gridlib.GRIDTABLE_NOTIFY_ROWS_DELETED,
                                         self.parent.GetNumberRows(), self.parent.GetNumberRows())
            self.pars=parameters.Parameters()
            self.GetView().ProcessTableMessage(msg)
            msg=gridlib.GridTableMessage(self,
                                         gridlib.GRIDTABLE_REQUEST_VIEW_GET_VALUES)
            self.GetView().ProcessTableMessage(msg)

            self.GetView().ForceRefresh()

            self.pars=pars
            msg=gridlib.GridTableMessage(self,
                                         gridlib.GRIDTABLE_NOTIFY_ROWS_APPENDED, self.pars.get_len_rows()+1)
            self.GetView().ProcessTableMessage(msg)

            msg=gridlib.GridTableMessage(self,
                                         gridlib.GRIDTABLE_REQUEST_VIEW_GET_VALUES)
            self.GetView().ProcessTableMessage(msg)

            self.GetView().ForceRefresh()
            # print 'In parametergrid ', self.pars.data
        else:
            self.pars=pars
            msg=gridlib.GridTableMessage(self,
                                         gridlib.GRIDTABLE_REQUEST_VIEW_GET_VALUES)
            self.GetView().ProcessTableMessage(msg)
        self.parent._grid_changed(permanent_change=permanent_change)

    def ChangeValueInteractively(self, row, value):
        """ Callback for a change of the value. Used to interactively set the value and notify other parts
        of GenX by posting a EVT_PARAMETER_VALUE_CHANGE event.

        """
        self.SetValue(row, 1, value)
        self.parent.PostValueChangedEvent()

# Class ParameterDataTable ends here
# ------------------------------------------------------------------------------
class SliderCellEditor(gridlib.GridCellEditor):

    def __init__(self, value=0.0, min_value=0.0, max_value=100.0):
        gridlib.GridCellEditor.__init__(self)
        self.value=value
        self.startValue=value
        self.min_value=min_value
        self.max_value=max_value

    def Create(self, parent, id, evtHandler):
        self._tc=ctrls.SliderControl(parent, id, value=self.value, max_value=self.max_value,
                                     min_value=self.min_value, font=parent.GetFont())
        self.SetControl(self._tc)

        if evtHandler:
            self._tc.PushEventHandler(evtHandler)

    def GetValue(self):
        return self._tc.GetValue()

    def SetSize(self, rect):
        """
        Called to position/size the edit control within the cell rectangle.
        """
        self._tc.SetSize(rect.x-1, rect.y-1, rect.width+2, rect.height+2,
                         wx.SIZE_ALLOW_MINUS_ONE)

    def StartingClick(self):
        """
        Make a correct action if editor activated by mouse click
        """
        # print "Staring click"
        pass

    def BeginEdit(self, row, col, grid):
        """
        Fetch the value from the table and prepare the edit control
        to begin editing.  Set the focus to the edit control.
        """

        self.startValue=grid.GetTable().GetValue(row, col)
        self.max_value=grid.GetTable().GetValue(row, col+3)
        self.min_value=grid.GetTable().GetValue(row, col+2)
        if self.startValue!='':
            self.startValue=float(self.startValue)
        else:
            self.startValue=0.0
            grid.GetTable().SetValue(row, col, self.startValue)

        if self.max_value!='':
            self.max_value=float(self.max_value)
        else:
            self.max_value=0.0
            grid.GetTable().SetValue(row, col, self.max_value)
        if self.min_value!='':
            self.min_value=float(self.min_value)
        else:
            self.min_value=0.0
            grid.GetTable().SetValue(row, col, self.min_value)

        self._tc.SetValue(float(self.startValue))
        self._tc.SetMaxValue(self.max_value)
        self._tc.SetMinValue(self.min_value)
        self._tc.bind_handlers()
        self._tc.SetScrollCallback(lambda val: grid.GetTable().ChangeValueInteractively(row, val))
        self._tc.SetFocus()
        # print "begin edit finished"

    def EndEdit(self, row, col, grid, oldVal):
        """
        End editing the cell.  This function must check if the current
        value of the editing control is valid and different from the
        original value (available as oldval in its string form.)  If
        it has not changed then simply return None, otherwise return
        the value in its string form.
        """
        # print "EndEdit"
        self._tc.unbind_handlers()
        val=self._tc.GetValue()
        self._tc.SetScrollCallback(None)
        if val!=oldVal:
            return val
        else:
            return None

    def ApplyEdit(self, row, col, grid):
        """
        This function  saves the value of the control into the
        grid or grid table.
        """
        val=self._tc.GetValue()
        grid.GetTable().SetValue(row, col, val)  # update the table

        self.startValue=float(val)
        self._tc.SetValue(self.startValue)

    def Reset(self):
        """
        Reset the value in the control back to its starting value.
        """
        self._tc.SetValue(self.startValue)

    def Clone(self):
        """
        Create a new object which is the copy of this one
        """
        return SliderCellEditor(value=self.value, min_value=self.min_value, max_value=self.max_value)

    def Destroy(self):
        """final cleanup"""
        self._tc.Destroy()
        super(SliderCellEditor, self).Destroy()


class ValueLimitCellEditor(gridlib.GridCellEditor):
    """Editor for the parameter values with a spin control"""

    def __init__(self, value=0.0, min_value=0.0, max_value=100.0, ticks=20, digits=5):
        gridlib.GridCellEditor.__init__(self)
        self.value=value
        self.startValue=value
        self.min_value=min_value
        self.max_value=max_value
        self.ticks=float(ticks)
        self.digits=digits

    def Create(self, parent, id, evtHandler):
        self._tc=ctrls.SpinCtrl(parent, id, value=self.value)
        self.SetControl(self._tc)

        if evtHandler:
            self._tc.PushEventHandler(evtHandler)

    def Show(self, show, attr):
        super(ValueLimitCellEditor, self).Show(show, attr)

    def GetValue(self):
        return self._tc.GetValue()

    def SetSize(self, rect):
        """Called to position/size the edit control within the cell rectangle."""
        self._tc.SetSize(rect.x-1, rect.y-1, rect.width+2, rect.height+2,
                         wx.SIZE_ALLOW_MINUS_ONE)

    def BeginEdit(self, row, col, grid):
        """ Fetch the value from the table and prepare the edit control to begin editing.  Set the focus to the
        edit control.
        """
        self.startValue=grid.GetTable().GetValue(row, col)
        self.max_value=grid.GetTable().GetValue(row, col+3)
        self.min_value=grid.GetTable().GetValue(row, col+2)
        if self.startValue!='':
            self.startValue=float(self.startValue)
        else:
            self.startValue=0.0
            grid.GetTable().SetValue(row, col, self.startValue)

        if self.max_value!='':
            self.max_value=float(self.max_value)
        else:
            self.max_value=0.0
            grid.GetTable().SetValue(row, col, self.max_value)
        if self.min_value!='':
            self.min_value=float(self.min_value)
        else:
            self.min_value=0.0
            grid.GetTable().SetValue(row, col, self.min_value)

        # self._tc.SetValue('%.7g'%(self.startValue))
        self._tc.SetValue(self.startValue)
        self._tc.SetRange(self.min_value, self.max_value)
        # self._tc.SetIncrement((self.max_value - self.min_value)/self.ticks)
        self._tc.SetValueChangeCallback(lambda val: grid.GetTable().ChangeValueInteractively(row, val))
        self._tc.SetFocus()

    def EndEdit(self, row, col, grid, oldVal):
        """
        End editing the cell.  This function must check if the current
        value of the editing control is valid and different from the
        original value (available as oldval in its string form.)  If
        it has not changed then simply return None, otherwise return
        the value in its string form.
        """
        val=float(self._tc.GetValue())
        # val = max(self.min_value, val)
        # val = min(self.max_value, val)
        # self._tc.SetValue('%.5g'%(val))
        # self._tc.SetValue(val)
        if self._tc.value_change_func is not None:
            self._tc.value_change_func(val)
        self._tc.SetValueChangeCallback(None)
        return float(val)

    def ApplyEdit(self, row, col, grid):
        """ This function  saves the value of the control into the grid or grid table."""
        val=self._tc.GetValue()
        grid.GetTable().SetValue(row, col, float(val))  # update the table

        self.startValue=float(val)
        # self._tc.SetValue('%.5g' % self.startValue)
        self._tc.SetValue(self.startValue)

    def IsAcceptedKey(self, evt):
        """
        Return True to allow the given key to start editing: the base class
        version only checks that the event has no modifiers.  F2 is special
        and will always start the editor.
        """
        key=evt.GetKeyCode()
        return chr(key) in (string.digits+'.- ')

    def StartingKey(self, evt):
        """
        If the editor is enabled by pressing keys on the grid, this will be
        called to let the editor do something about that first key if desired.
        """
        key=evt.GetKeyCode()
        ch=None
        if key in [wx.WXK_NUMPAD0, wx.WXK_NUMPAD1, wx.WXK_NUMPAD2, wx.WXK_NUMPAD3,
                   wx.WXK_NUMPAD4, wx.WXK_NUMPAD5, wx.WXK_NUMPAD6, wx.WXK_NUMPAD7,
                   wx.WXK_NUMPAD8, wx.WXK_NUMPAD9
                   ]:
            ch=chr(ord('0')+key-wx.WXK_NUMPAD0)

        elif 256>key>=0 and chr(key) in string.printable:
            ch=chr(key)

        if ch in (string.digits+'.-'):
            # For this example, replace the text.  Normally we would append it.
            # self._tc.AppendText(ch)
            self._tc.SetValue(ch)
            self._tc.SetInsertionPointEnd()
        else:
           evt.Skip()

    def Reset(self):
        """Reset the value in the control back to its starting value."""
        self._tc.SetValue('%.5g'%self.startValue)

    def Clone(self):
        """
        Create a new object which is the copy of this one
        """
        return ValueLimitCellEditor(value=self.value, min_value=self.min_value, max_value=self.max_value)

class ValueLimitCellRenderer(gridlib.GridCellRenderer):
    """ Renderer for the Parameter Values. Colours the Cell if the value is out of bounds.
    """

    def __init__(self, value=0, max_value=100.0, min_value=100):
        gridlib.GridCellRenderer.__init__(self)

    def Draw(self, grid, attr, dc, rect, row, col, isSelected):
        if grid.GetCellValue(row, col)!='':
            val=float(grid.GetCellValue(row, col))
            min_val=float(grid.GetCellValue(row, col+2))
            max_val=float(grid.GetCellValue(row, col+3))
            if val>max_val or val<min_val:
                bkg_colour=wx.Colour(204, 0, 0)
                txt_colour=wx.Colour(255, 255, 255)
            else:
                if not isSelected:
                    bkg_colour=attr.GetBackgroundColour()
                else:
                    bkg_colour=grid.GetSelectionBackground()
                txt_colour=wx.Colour(0, 0, 0)

            dc.SetBackgroundMode(wx.SOLID)
            dc.SetBrush(wx.Brush(bkg_colour, wx.SOLID))
            dc.SetPen(wx.TRANSPARENT_PEN)
            dc.SetClippingRegion(rect.x, rect.y, rect.width, rect.height)
            dc.DrawRectangle(rect.x, rect.y, rect.width, rect.height)

            text='%.7g'%val

            if val<=max_val and val>=min_val:
                # paint a slight indication of rlative value within range
                dc.SetBrush(wx.Brush(wx.Colour(240, 240, 240), wx.SOLID))
                if max_val!=min_val:
                    rel_value_pix=int(rect.width*(val-min_val)/(max_val-min_val))
                else:
                    rel_value_pix=0
                dc.DrawRectangle(rect.x, rect.y, rel_value_pix, rect.height)

            dc.SetTextForeground(txt_colour)
            dc.SetTextBackground(bkg_colour)
            dc.SetFont(attr.GetFont())
            width, height=dc.GetTextExtent(text)
            dc.DrawText(text, rect.x+rect.width-width-1, rect.y+1)

            dc.DestroyClippingRegion()
        else:
            dc.SetBackgroundMode(wx.SOLID)
            if isSelected:
                dc.SetBrush(wx.Brush(grid.GetSelectionBackground(), wx.SOLID))
            else:
                dc.SetBrush(wx.Brush(attr.GetBackgroundColour(), wx.SOLID))
            dc.SetPen(wx.TRANSPARENT_PEN)
            dc.DrawRectangle(rect.x, rect.y, rect.width, rect.height)

    def Clone(self):
        return ValueLimitCellRenderer(self.slider_drawer.value, max=self.slider_drawer.max_value,
                                  min=self.slider_drawer.min_value)

class ValueCellEditor(gridlib.GridCellEditor):
    """Editor for the parameter values with a spin control"""

    def __init__(self, value=0.0, digits=5):
        gridlib.GridCellEditor.__init__(self)
        self.value=value
        self.startValue=value
        # self.min_value = min_value
        # self.max_value = max_value
        # self.ticks = float(ticks)
        self.digits=digits

    def Create(self, parent, id, evtHandler):
        self._tc=wx.TextCtrl(parent, id, style=wx.TE_RIGHT, validator=ctrls.NumberValidator())
        # self._tc = ctrls.SpinCtrl(parent, id, value=self.value)
        self.SetControl(self._tc)

        if evtHandler:
            self._tc.PushEventHandler(evtHandler)

    def GetValue(self):
        return float(self._tc.GetValue())

    def SetSize(self, rect):
        """Called to position/size the edit control within the cell rectangle."""
        self._tc.SetSize(rect.x-1, rect.y-1, rect.width+2, rect.height+2, sizeFlags=wx.SIZE_ALLOW_MINUS_ONE)

    def BeginEdit(self, row, col, grid):
        """ Fetch the value from the table and prepare the edit control to begin editing.  Set the focus to the
        edit control.
        """
        self.startValue=grid.GetTable().GetValue(row, col)
        if self.startValue!='':
            self.startValue=float(self.startValue)
        else:
            self.startValue=0.0
            grid.GetTable().SetValue(row, col, self.startValue)

        self._tc.SetValue('%.7g'%self.startValue)
        # self._tc.SetValue(self.startValue)
        # self._tc.SetValueChangeCallback(lambda val: grid.GetTable().ChangeValueInteractively(row, val))
        self._tc.SetFocus()

    def EndEdit(self, row, col, grid, oldVal):
        """
        End editing the cell.  This function must check if the current
        value of the editing control is valid and different from the
        original value (available as oldval in its string form.)  If
        it has not changed then simply return None, otherwise return
        the value in its string form.
        """
        val=float(self._tc.GetValue())
        # val = max(self.min_value, val)
        # val = min(self.max_value, val)
        # self._tc.SetValue('%.5g'%(val))
        # self._tc.SetValue(val)
        # self._tc.value_change_func(val)
        # self._tc.SetValueChangeCallback(None)
        return float(val)

    def ApplyEdit(self, row, col, grid):
        """ This function  saves the value of the control into the grid or grid table."""
        val=self._tc.GetValue()
        grid.GetTable().SetValue(row, col, float(val))  # update the table

        self.startValue=float(val)
        self._tc.SetValue('%.7g'%self.startValue)
        # self._tc.SetValue(self.startValue)

    def IsAcceptedKey(self, evt):
        """
        Return True to allow the given key to start editing: the base class
        version only checks that the event has no modifiers.  F2 is special
        and will always start the editor.
        """
        key=evt.GetKeyCode()
        return chr(key) in (string.digits+'.- ')

    def StartingKey(self, evt):
        """
        If the editor is enabled by pressing keys on the grid, this will be
        called to let the editor do something about that first key if desired.
        """
        key=evt.GetKeyCode()
        ch=None
        if key in [wx.WXK_NUMPAD0, wx.WXK_NUMPAD1, wx.WXK_NUMPAD2, wx.WXK_NUMPAD3,
                   wx.WXK_NUMPAD4, wx.WXK_NUMPAD5, wx.WXK_NUMPAD6, wx.WXK_NUMPAD7,
                   wx.WXK_NUMPAD8, wx.WXK_NUMPAD9
                   ]:
            ch=chr(ord('0')+key-wx.WXK_NUMPAD0)

        elif 256>key>=0 and chr(key) in string.printable:
            ch=chr(key)

        if ch in (string.digits+'.-'):
            # For this example, replace the text.  Normally we would append it.
            # self._tc.AppendText(ch)
            self._tc.SetValue(ch)
            self._tc.SetInsertionPointEnd()
        else:
            evt.Skip()

    def Reset(self):
        """Reset the value in the control back to its starting value."""
        self._tc.SetValue('%.5g'%self.startValue)

    def Clone(self):
        """
        Create a new object which is the copy of this one
        """
        return ValueCellEditor(value=self.value, min_value=self.min_value, max_value=self.max_value)

class ValueCellRenderer(gridlib.GridCellRenderer):
    """ Renderer for the Parameter Values. Colours the Cell if the value is out of bounds.
    """

    def __init__(self, value=0, max_value=100.0, min_value=100):
        gridlib.GridCellRenderer.__init__(self)

    def Draw(self, grid, attr, dc, rect, row, col, isSelected):
        if grid.GetCellValue(row, col)!='':
            val=float(grid.GetCellValue(row, col))
            if not isSelected:
                bkg_colour=attr.GetBackgroundColour()
            else:
                bkg_colour=grid.GetSelectionBackground()
            txt_colour=wx.Colour(0, 0, 0)

            dc.SetBackgroundMode(wx.SOLID)
            dc.SetBrush(wx.Brush(bkg_colour, wx.SOLID))
            dc.SetPen(wx.TRANSPARENT_PEN)
            dc.SetClippingRegion(rect.x, rect.y, rect.width, rect.height)
            dc.DrawRectangle(rect.x, rect.y, rect.width, rect.height)

            text='%.7g'%val

            dc.SetTextForeground(txt_colour)
            dc.SetTextBackground(bkg_colour)
            dc.SetFont(attr.GetFont())
            width, height=dc.GetTextExtent(text)
            dc.DrawText(text, rect.x+rect.width-width-1, rect.y+1)

            dc.DestroyClippingRegion()
        else:
            dc.SetBackgroundMode(wx.SOLID)
            if isSelected:
                dc.SetBrush(wx.Brush(grid.GetSelectionBackground(), wx.SOLID))
            else:
                dc.SetBrush(wx.Brush(attr.GetBackgroundColour(), wx.SOLID))
            dc.SetPen(wx.TRANSPARENT_PEN)
            dc.DrawRectangle(rect.x, rect.y, rect.width, rect.height)

    def Clone(self):
        return ValueCellRenderer(self.slider_drawer.value, max=self.slider_drawer.max_value,
                                  min=self.slider_drawer.min_value)

# ------------------------------------------------------------------------------
class ParameterGrid(wx.Panel):
    '''
    The GUI component itself. This is the thing to use in a GUI.
    '''

    def __init__(self, parent, frame, config=None):
        wx.Panel.__init__(self, parent)

        self.config=config
        self.config_name='parameter grid'

        # The two main widgets
        self.toolbar=wx.ToolBar(self, style=wx.TB_FLAT | wx.TB_VERTICAL)
        self.grid=gridlib.Grid(self, -1, style=wx.NO_BORDER)
        self.grid._grid_changed=self._grid_changed
        self.grid.PostValueChangedEvent=self.PostValueChangedEvent
        self.grid.DisableDragColSize()
        self.grid.DisableDragRowSize()
        # self.grid.SetForegroundColour('BLUE')

        self.do_toolbar()

        self.sizer_hor=wx.BoxSizer(wx.HORIZONTAL)
        self.SetSizer(self.sizer_hor)
        self.sizer_hor.Add(self.toolbar, proportion=0, flag=wx.EXPAND, border=0)
        self.sizer_hor.Add(self.grid, proportion=1, flag=wx.EXPAND, border=0)

        self.parent=frame
        self.prt=printout.PrintTable(parent)

        self.project_func=None
        self.scan_func=None

        self.table=ParameterDataTable(self.grid)

        self.variable_span=0.25
        # The functions has to begin with the following letters:
        self.set_func='set'
        self.get_func='get'
        # The second parameter means that the grid is to take ownership of the
        # table and will destroy it when done.  Otherwise you would need to keep
        # a reference to it and call it's Destroy method later.
        self.grid.SetTable(self.table, True)
        self.grid.SetSelectionMode(gridlib.Grid.SelectCells)
        # self.grid.SetSelectionBackground()

        self.grid.SetRowLabelSize(50)
        self.grid.SetMargins(0, 0)
        # This is the my original column True means set as min...
        # self.AutoSizeColumns(True)
        # The new
        self.grid.AutoSizeColumn(0, False)
        self.grid.AutoSizeColumn(1, False)
        self.grid.AutoSizeColumn(2, True)
        self.grid.AutoSizeColumn(3, False)
        self.grid.AutoSizeColumn(4, False)
        self.grid.AutoSizeColumn(5, False)

        self.par_dict={}

        self.grid.GetGridWindow().Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.grid.Bind(gridlib.EVT_GRID_CELL_LEFT_DCLICK, self.OnLeftDClick)
        self.grid.Bind(gridlib.EVT_GRID_CMD_CELL_LEFT_CLICK, self.OnLeftClick)
        self.grid.Bind(gridlib.EVT_GRID_CMD_CELL_RIGHT_CLICK, self.OnRightClick)
        self.grid.Bind(gridlib.EVT_GRID_LABEL_RIGHT_CLICK, self.OnLabelRightClick)
        self.grid.Bind(wx.EVT_SIZE, self.OnResize)
        self.grid.Bind(gridlib.EVT_GRID_SELECT_CELL, self.OnSelectCell)
        self.grid.Bind(wx.EVT_CHAR, self.OnKey)

        self.toolbar.Realize()
        self.show_slider=False
        self.SetValueEditorSlider(slider=False)
        attr=gridlib.GridCellAttr()
        attr.SetEditor(ValueCellEditor())
        attr.SetRenderer(ValueCellRenderer())
        self.grid.SetColAttr(3, attr.Clone())
        self.grid.SetColAttr(4, attr)
        self._paste_history=None

    def PrepareNewModel(self):
        """ Hack to prepare the grid for a new model.
        :return:
        """
        # This hack is needed to deselect any current cell. Have not found a better way to solve it.
        # If not called the program can cause an segmentation fault and crash.
        self.grid.SetGridCursor(0, 3)
        self.grid.SetGridCursor(0, 4)

    def ReadConfig(self):
        """ Reads the variables stored in the config file."""
        try:
            val=self.config.get_boolean(self.config_name, 'value slider')
        except filehandling.OptionError:
            iprint('Could not locate option %s.%s'%(self.config_name, 'y scale'))
            self.SetValueEditorSlider(False)
        else:
            self.SetValueEditorSlider(val)

    def WriteConfig(self, show_slider=None):
        """Writes the varaibles to be stored to the config"""
        if show_slider is not None:
            self.config.set(self.config_name, 'value slider', show_slider)

    def OnKey(self, event):
        if event.ControlDown() and event.GetKeyCode() == 22:
            self.OnPaste()
        else:
            event.Skip()

    def OnPaste(self):
        clipboard=wx.TextDataObject()
        if wx.TheClipboard.Open():
            wx.TheClipboard.GetData(clipboard)
            wx.TheClipboard.Close()
        else:
            wx.MessageBox("Can't open the clipboard", "Error")
        data=list(map(str.split, clipboard.Text.splitlines()))
        sblocks=list(self.grid.GetSelectedBlocks())
        if len(sblocks)==0:
            start_row=self.grid.GetGridCursorRow()
            start_col=self.grid.GetGridCursorCol()
            srows=list(range(start_row, self.grid.GetNumberRows()))
            scols=list(range(start_col, self.grid.GetNumberCols()))
        elif len(sblocks)>1:
            # don't paste if individual blocks have been selected
            return
        else:
            sblock=sblocks[0]
            start_row=sblock.TopRow
            start_col=sblock.LeftCol
            srows=list(range(start_row, sblock.BottomRow+1))
            scols=list(range(start_col, sblock.RightCol+1))

        self._paste_history=[start_row, start_col, []]
        for i, di in enumerate(data):
            self._paste_history[-1].append([])
            for j, dj in enumerate(di):
                row, col=i+start_row, j+start_col
                if row in srows and col in scols:
                    self._paste_history[-1][-1].append(self.table.GetValue(row, col))
                    if col in [1,3,4]:
                        try:
                            dj=float(dj)
                        except ValueError:
                            continue
                    elif col==2:
                        try:
                            dj=bool(dj)
                        except ValueError:
                            continue
                    self.table.SetValue(row, col, dj)

    def SetValueEditorSlider(self, slider=True):
        """ Set the Editor and Renderer as slider instead of text.

        :param slider: Flag determining if the Editor (and Renderer) should be a slider.
        :return:
        """
        # print "SetValueEditorSlider"
        row=self.grid.GetGridCursorRow()
        col=self.grid.GetGridCursorCol()
        # print row, col
        # print self.grid.GetSelectedCells()
        # This will disable the editor in the cell that currently is under editing.
        # If the editor is active the program will crash when changing the cell editor.
        self.grid.SetGridCursor(0, 3)
        self.grid.SetGridCursor(0, 4)
        self.grid.EnableCellEditControl(False)
        attr=gridlib.GridCellAttr()
        if slider:
            attr.SetEditor(SliderCellEditor())
            attr.SetRenderer(ValueLimitCellRenderer())
        else:
            attr.SetEditor(ValueLimitCellEditor())
            attr.SetRenderer(ValueLimitCellRenderer())
        self.grid.SetColAttr(1, attr)
        self.grid.SetGridCursor(row, col)
        self.show_slider=slider
        self.WriteConfig(show_slider=slider)

    def GetValueEditorSlider(self):
        """Returns True if the slider editor is active"""
        return self.show_slider

    def PostValueChangedEvent(self):
        evt=value_change()
        wx.PostEvent(self.parent, evt)

    def do_toolbar(self):
        # self.toolbar.SetToolBitmapSize((21,21))
        # self.toolbar.SetToolSeparation(5)
        # self.toolbar.SetBackgroundStyle(wx.BG_STYLE_COLOUR)
        # self.toolbar.SetBackgroundColour(wx.SystemSettings.GetColour(wx.SYS_COLOUR_MENUBAR))
        # self.toolbar.SetBackgroundColour('BLUE')
        dpi_scale_factor=wx.GetApp().dpi_scale_factor
        tb_bmp_size=int(dpi_scale_factor*20)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, label='Add a new row',
                             bitmap=wx.Bitmap(img.add.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Insert new row')
        self.Bind(wx.EVT_TOOL, self.eh_add_row, id=newid)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, label='Delete row',
                             bitmap=wx.Bitmap(img.delete.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Delete row')
        self.Bind(wx.EVT_TOOL, self.eh_delete_row, id=newid)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, label='Move row up',
                             bitmap=wx.Bitmap(img.move_up.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Move row up')
        self.Bind(wx.EVT_TOOL, self.eh_move_row_up, id=newid)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, label='Move row down',
                             bitmap=wx.Bitmap(img.move_down.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Move row down')
        self.Bind(wx.EVT_TOOL, self.eh_move_row_down, id=newid)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, label='Sort parameters',
                             bitmap=wx.Bitmap(img.sort.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Sort the rows by class, object and name')
        self.Bind(wx.EVT_TOOL, self.eh_sort, id=newid)

        self.toolbar.AddSeparator()

        newid=wx.NewId()
        self.slider_tool_id=newid
        self.slider_tool=self.toolbar.AddCheckTool(newid, label='Show sliders', bitmap1=wx.Bitmap(
            img.slider.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                                                   shortHelp='Show the parameter values as sliders')
        self.Bind(wx.EVT_TOOL, self.eh_slider_toggle, id=newid)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, label='Project FOM evals',
                             bitmap=wx.Bitmap(img.par_proj.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Project FOM on parameter axis')
        self.Bind(wx.EVT_TOOL, self.eh_project_fom, id=newid)

        newid=wx.NewId()
        self.toolbar.AddTool(newid, label='Scan parameter',
                             bitmap=wx.Bitmap(img.par_scan.GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                             shortHelp='Scan FOM')
        self.Bind(wx.EVT_TOOL, self.eh_scan_fom, id=newid)

    def eh_add_row(self, event):
        """ Event handler for adding a row

        :param event:
        :return:
        """
        row=self.grid.GetGridCursorRow()
        self.table.InsertRow(row)

    def eh_delete_row(self, event):
        """Event handler for deleteing a row

        :param event:
        :return:
        """
        row=self.grid.GetGridCursorRow()
        self.table.DeleteRows([row, ])

    def eh_move_row_up(self, event):
        """ Event handler for moving a row up.

        :param event:
        :return:
        """
        row=self.grid.GetGridCursorRow()
        if self.table.MoveRowUp(row):
            self.grid.SetGridCursor(row-1, self.grid.GetGridCursorCol())

    def eh_move_row_down(self, event):
        """ Event handler for moving a row down.

        :param event:
        :return:
        """
        row=self.grid.GetGridCursorRow()
        if self.table.MoveRowDown(row):
            self.grid.SetGridCursor(row+1, self.grid.GetGridCursorCol())

    def eh_slider_toggle(self, event):
        """ Event handler for showing the sliders instead of values

        :param event:
        :return:
        """
        new_state=not self.GetValueEditorSlider()
        self.SetValueEditorSlider(new_state)
        self.parent.main_frame_menubar.mb_view_grid_slider.Check(new_state)
        self.Refresh()

    def toggle_slider_tool(self, state):
        self.toolbar.ToggleTool(self.slider_tool_id, state)

    def get_toggle_slider_tool_state(self):
        self.toolbar.GetToolState(self.slider_tool_id)

    def eh_project_fom(self, event):
        """ Event handler for toolbar project fom
        :param event:
        :return:
        """
        row=self.grid.GetGridCursorRow()
        if self.project_func:
            self.project_func(row)

    def eh_scan_fom(self, event):
        """ Event handler for scanning fom on selected parameter

        :param event:
        :return:
        """
        row=self.grid.GetGridCursorRow()
        if self.scan_func:
            self.scan_func(row)

    def eh_sort(self, event):
        """Event handler for the sorting

        :param event:
        :return:
        """
        self.table.SortRows()

    def OnSelectCell(self, evt):
        # row=evt.GetRow()
        # if row>-1:
        #     self.grid.SelectRow(row)
        evt.Skip()

    def _grid_changed(self, permanent_change=True):
        '''_grid_changed(self) --> None
        
        internal function to yield a EVT_PARAMETER_GRID_CHANGE
        '''
        self.grid.ForceRefresh()
        evt=grid_change()
        evt.permanent_change=permanent_change
        wx.PostEvent(self.parent, evt)

    def _update_printer(self):
        '''_update_printer(self) --> None
        
        Update the printer to have the same values as in the grid.
        '''
        data=[]
        for row in self.GetParameters().get_data():
            # data.append([' %.30s'%row[0],' %.5f'%row[1],\
            # ' %d'%row[2],' %.5g'%row[3],' %.5g'%row[4],' '+row[5]])
            data.append([row[0], '%.5f'%row[1],
                         '%d'%row[2], '%.5f'%row[3], '%.5f'%row[4], row[5]])
        self.prt.data=data
        self.prt.label=self.GetParameters().get_col_headers()
        self.prt.SetPaperId(wx.PAPER_A4)
        self.prt.SetLandscape()
        self.prt.page_width=11.69
        self.prt.page_height=8.26
        self.prt.cell_right_margin=0
        self.prt.cell_left_margin=0.05
        self.prt.text_font={"Name": 'Arial', "Size": 14, "Colour": [0, 0, 0], "Attr": [0, 0, 0]}
        self.prt.label_font={"Name": 'Arial', "Size": 24, "Colour": [0, 0, 0], "Attr": [0, 0, 0]}
        self.prt.set_column=[3, 1.8, 0.5, 1.5, 1.5, 1.5]
        self.prt.vertical_offset=0
        self.prt.horizontal_offset=0
        # self.prt.SetRowSpacing(0,0)
        self.prt.SetCellText(4, 2, wx.NamedColour('RED'))
        self.prt.SetHeader("Fittting parameters", align=wx.ALIGN_CENTRE, colour=wx.NamedColour('RED'), size=14)
        self.prt.SetFooter("Print Date/Time: ", type="Date & Time", align=wx.ALIGN_CENTRE, indent=1,
                           colour=wx.NamedColour('RED'), size=14)

    def Print(self):
        '''Print(self) --> None
        
        Prints the values to the printer
        '''
        pd=wx.PrintData()
        pd.SetPrinterName('')
        pd.SetOrientation(wx.LANDSCAPE)
        pd.SetPaperId(wx.PAPER_A4)
        pd.SetQuality(wx.PRINT_QUALITY_DRAFT)
        pd.SetColour(True)
        pd.SetNoCopies(1)
        pd.SetCollate(True)

        pdd=wx.PrintDialogData()
        pdd.SetPrintData(pd)
        pdd.SetMinPage(1)
        pdd.SetMaxPage(1)
        pdd.SetFromPage(1)
        pdd.SetToPage(2)
        pdd.SetPrintToFile(True)

        # printer = wx.Printer(pdd)
        self._update_printer()
        self.prt.Print()
        # prtout = printout.SetPrintout(self.prt)
        # printer.Print(self.parent, prtout, True)

    def PrintPreview(self):
        '''PrintPreview(self) --> None
        
        Prints a preview of the grid
        '''
        self.prt.Preview()

    def OnNewModel(self, evt):
        '''
        OnNewModel(self, evt) --> None
        
        callback for when a new model has been loaded or created 
        - reload all the parameters
        '''
        # Set the parameters in the table 
        # this also updates the grid. 
        self.table.SetParameters(evt.GetModel().get_parameters(),
                                 permanent_change=False)
        # Let the event proceed to other fucntions that have signed up.
        evt.Skip()

    def SetParameters(self, pars):
        self.table.SetParameters(pars)

    def OnSolverUpdateEvent(self, evt):
        '''OnSolverUpdateEvent(self, evt) --> None
        
        Callback to update the values in the grid from the optimizer
        Assumes that evt holds the following members:
        values: An array of the appropriate length (same as the number of 
                checked parameters to fit)
        new_best: A boolean indicating if there are a new best.
        '''
        if evt.new_best:
            # print evt.fitting
            self.table.pars.set_value_pars(evt.values)
            self.table.SetParameters(self.table.pars, clear=False,
                                     permanent_change=evt.permanent_change)

        evt.Skip()

    def OnLeftDClick(self, evt):
        """ Event handler that starts editing the cells on a double click and not
        just a second click as is default.

        :param evt: The Event passed to the event handler
        :return: Nothing
        """
        # if self.grid.CanEnableCellControl() and evt.GetCol() != 2:
        #    self.grid.EnableCellEditControl()
        ##self.grid.SelectRow(evt.GetRow())
        col, row=evt.GetCol(), evt.GetRow()
        if col==0 and row>-1:
            if self.grid.GetGridCursorRow()==row:
                self.CurSelection=(row, col)
                self.grid.SetGridCursor(row, col)
                pos=evt.GetPosition()
                self.show_parameter_menu(pos)
            else:
                evt.Skip()
        else:
            # if self.grid.CanEnableCellControl() and evt.GetCol() != 2:
            #    self.grid.EnableCellEditControl()
            evt.Skip()

    def OnLeftClick(self, evt):
        """Callback to handle left clicks in the cells.

        :param evt: A Event from a Grid
        :return: Nothing
        """
        col, row=evt.GetCol(), evt.GetRow()
        # self.grid.SelectRow(row)
        if col==2 and row>-1:
            self.table.SetValue(row, col, not self.table.GetValue(row, col))

        elif col==0 and row>-1:
            if self.grid.GetGridCursorRow()==row and self.table.GetValue(row, col)=='':
                self.CurSelection=(row, col)
                self.grid.SetGridCursor(row, col)
                pos=evt.GetPosition()
                self.show_parameter_menu(pos)
            else:
                evt.Skip()
        # elif col==1 and row>-1:
        #     if self.show_slider:
        #         self.grid.SetGridCursor(row, col)
        #         self.grid.EnableCellEditControl()
        #     else:
        #         evt.Skip()
        else:
            evt.Skip()

    def OnLeftDown(self, event):
        """ Callback for left down - handling slider activation

        :param evt:
        :return:
        """
        # print "Left Down"
        x, y=self.grid.CalcUnscrolledPosition(event.GetX(), event.GetY())
        row, col=self.grid.XYToCell(x, y)
        if col==1 and row>-1:
            # print "Activating editor"
            # self.grid.SetGridCursor(row, col)
            # wx.CallAfter(self.grid.EnableCellEditControl)#self.grid.EnableCellEditControl()
            # self.grid.ShowCellEditControl()
            event.Skip()
        else:
            event.Skip()

    def show_label_menu(self, row):
        insertID=wx.NewId()
        deleteID=wx.NewId()
        projectID=wx.NewId()
        scanID=wx.NewId()
        menu=wx.Menu()
        menu.Append(insertID, "Insert Row")
        menu.Append(deleteID, "Delete Row(s)")
        # Item is checked for fitting
        if self.table.GetValue(row, 2):
            menu.Append(projectID, "Project FOM")
        if self.table.GetValue(row, 0)!='':
            menu.Append(scanID, "Scan FOM")

        def insert(event, self=self, row=row):
            self.table.InsertRow(row)

        def delete(event, self=self, row=row):
            rows=self.grid.GetSelectedRows()
            self.table.DeleteRows(rows)

        def projectfom(event, self=self, row=row):
            if self.project_func:
                self.project_func(row)

        def scanfom(event, self=self, row=row):
            if self.scan_func:
                self.scan_func(row)

        self.Bind(wx.EVT_MENU, insert, id=insertID)
        self.Bind(wx.EVT_MENU, delete, id=deleteID)
        # Item is checked for fitting
        if self.table.GetValue(row, 2):
            self.Bind(wx.EVT_MENU, projectfom, id=projectID)
        if self.table.GetValue(row, 0)!='':
            self.Bind(wx.EVT_MENU, scanfom, id=scanID)
        self.PopupMenu(menu)
        menu.Destroy()

    def OnLabelRightClick(self, evt):
        """ Callback function that opens a popupmenu for appending or
        deleting rows in the grid.
        """
        # check so a row is selected
        col, row=evt.GetCol(), evt.GetRow()
        if col==-1:
            if not self.grid.GetSelectedRows():
                self.grid.SelectRow(row)
            # making the menus
            self.show_label_menu(row)

    def SetFOMFunctions(self, projectfunc, scanfunc):
        '''SetFOMFunctions(self, projectfunc, scanfunc) --> None
        
        Set the functions for executing the projection and scan function.
        '''
        self.project_func=projectfunc
        self.scan_func=scanfunc

    def show_parameter_menu(self, pos):
        self.pmenu=wx.Menu()
        par_dict=self.par_dict
        classes=list(par_dict.keys())
        classes.sort(key=str.lower)
        for cl in classes:
            # Create a submenu for each class
            clmenu=wx.Menu()
            if isinstance(par_dict[cl], dict):
                obj_dict=par_dict[cl]
                objs=list(obj_dict.keys())
                objs.sort(key=str.lower)
                # Create a submenu for each object
                for obj in objs:
                    obj_menu=wx.Menu()
                    funcs=obj_dict[obj]
                    funcs.sort(key=str.lower)
                    # Create an item for each method
                    for func in funcs:
                        item=obj_menu.Append(-1, obj+'.'+func)
                        self.Bind(wx.EVT_MENU, self.OnPopUpItemSelected, item)
                    clmenu.Append(-1, obj, obj_menu)
                # self.pmenu.Append(-1, cl, clmenu)
            elif isinstance(par_dict[cl], list):
                objs=par_dict[cl]
                objs.sort(key=str.lower)
                # Create an item for each method
                for obj in objs:
                    item=clmenu.Append(-1, obj)
                    self.Bind(wx.EVT_MENU, self.OnPopUpItemSelected, item)
            self.pmenu.Append(-1, cl, clmenu)

        # Check if there are no available classes
        if len(classes)==0:
            # Add an item to compile the model
            item=self.pmenu.Append(-1, 'Simulate to see parameters')
            self.Bind(wx.EVT_MENU, self.OnPopUpItemSelected, item)
        # Add an item for edit the cell manually
        item=self.pmenu.Append(-1, 'Manual Edit')
        self.Bind(wx.EVT_MENU, self.OnPopUpItemSelected, item)

        self.PopupMenu(self.pmenu, pos)
        self.pmenu.Destroy()

    def OnRightClick(self, evt):
        ''' Callback function that creates a popupmenu when a row in the
        first column is clicked. The menu contains all parameters that are 
        selectable and fitable. 
        '''
        # print dir(evt)
        col=evt.GetCol()
        row=evt.GetRow()
        # print row,col
        if col==0:
            self.CurSelection=(row, col)
            self.grid.SetGridCursor(row, col)
            pos=evt.GetPosition()
            self.show_parameter_menu(pos)

    def OnPopUpItemSelected(self, event):
        """ Callback for the popmenu to select parameter to fit.
        When a item from the OnRightClick is selected this function takes care
        of setting:
        1. the parametername
        2. The current value of the parameter in Value
        3. The min and max values

        :param event: event from the menu
        :return: Nothing
        """
        item=self.pmenu.FindItemById(event.GetId())
        # Check if the item should be edit manually
        if item.GetItemLabel()=='Simulate to see parameters':
            self.parent.eh_tb_simulate(event)
            return
        if item.GetItemLabel()=='Manual Edit':
            if self.grid.CanEnableCellControl():
                self.grid.EnableCellEditControl()
            return
        # GetItemLabel seems to screw up underscores a bit replacing the with a
        # double one - this fixes it
        text=item.GetItemLabel().replace('__', '_')
        self.grid.SetCellValue(self.CurSelection[0], self.CurSelection[1], text)
        # Try to find out if the values also has an get function so that
        # we can init the value to the default!
        lis=text.split('.'+self.set_func)
        if len(lis)==2:
            try:
                value=self.evalf(lis[0]+'.'+self.get_func+lis[1])().real
                self.table.SetValue(self.CurSelection[0], 1, value)
                # Takes care so that also negative numbers give the
                # correct values in the min and max cells
                minval=value*(1-self.variable_span)
                maxval=value*(1+self.variable_span)
                self.table.SetValue(self.CurSelection[0], 3,
                                    min(minval, maxval))
                self.table.SetValue(self.CurSelection[0], 4,
                                    max(minval, maxval))
            except Exception as S:
                iprint("Not possible to init the variable automatically")
                # print S
        else:
            # It could be a Parameter class
            try:
                value=self.evalf(text+'.value')
                self.table.SetValue(self.CurSelection[0], 1, value)
                # Takes care so that also negative numbers give the
                # correct values in the min and max cells
                minval=value*(1-self.variable_span)
                maxval=value*(1+self.variable_span)
                self.table.SetValue(self.CurSelection[0], 3,
                                    min(minval, maxval))
                self.table.SetValue(self.CurSelection[0], 4,
                                    max(minval, maxval))
            except Exception as S:
                iprint("Not possible to init the variable automatically")
                # print S

    def OnResize(self, evt):
        self.SetColWidths()
        evt.Skip()

    def SetParameterSelections(self, par_dict):  # objlist,funclist):
        '''SetParameterSelections(self,objlist,funclist) --> None
        
        Function to set which parameter function that are chooseable 
        in the popup menu. objlist and funclist are lists of strings which
        has to be of the same size
        '''
        self.par_dict=par_dict

    def SetEvalFunc(self, func):
        '''
        Sets the fucntion that evaluates the expression that is exexuted in the
        model. The fucntion should take a string as input.
        '''
        self.evalf=func

    def SetColWidths(self):
        '''
        Function automatically set the cells width in the Grid to 
        reasonable values.
        '''
        width=(self.grid.GetSize().GetWidth()-self.grid.GetColSize(2)-self.grid.GetRowLabelSize())/5-1
        # To avoid warnings relating to a width < 0. This can occur during startup
        if width<=0:
            width=1
        self.grid.SetColSize(0, width)
        self.grid.SetColSize(1, width)
        self.grid.SetColSize(3, width)
        self.grid.SetColSize(4, width)
        self.grid.SetColSize(5, width)

    def GetParameters(self):
        '''
        Function that returns the parameters - Is this needed anymore?
        '''
        return self.table.pars

# ==============================================================================
# Custom events needed for updating and message parsing between the different
# modules.

# Event for when the grid has new values
(grid_change, EVT_PARAMETER_GRID_CHANGE)=wx.lib.newevent.NewEvent()
# Event for then the value of a parameter has changed. Should be used to do
# simulations interactively.
(value_change, EVT_PARAMETER_VALUE_CHANGE)=wx.lib.newevent.NewEvent()
